@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 // Taskwarrior Lua API
 //
-// Copyright 2006 - 2010, Paul Beckingham.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -115,13 +115,6 @@ static int api_task_feature (lua_State* L)
 #endif
   }
 
-  else if (name == "ncurses")
-  {
-#ifdef HAVE_NCURSES
-     value = true;
-#endif
-  }
-
   else if (name == "lua")
     value = true;
 
