@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -27,7 +27,7 @@
 #ifndef INCLUDED_API
 #define INCLUDED_API
 
-#include "auto.h"
+#include "../auto.h"
 #ifdef HAVE_LIBLUA
 
 #include <vector>
