@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -29,13 +29,15 @@
 #include <sstream>
 #include <stdlib.h>
 #include <string.h>
-#include "text.h"
-#include "Color.h"
-#include "util.h"
-#include "Date.h"
-#include "Duration.h"
-#include "Context.h"
-#include "Att.h"
+#include <text.h>
+#include <rx.h>
+#include <Color.h>
+#include <util.h>
+#include <Date.h>
+#include <Duration.h>
+#include <Context.h>
+#include <Att.h>
+#include <main.h>
 
 extern Context context;
 
@@ -52,6 +54,7 @@ static const char* internalNames[] =
   "status",
   "description",
   "tags",
+  "urgency",
   // Note that annotations are not listed.
 };
 
@@ -89,6 +92,22 @@ static const char* modifierNames[] =
 #define NUM_MODIFIABLE_NAMES (sizeof (modifiableNames) / sizeof (modifiableNames[0]))
 #define NUM_MODIFIER_NAMES   (sizeof (modifierNames)   / sizeof (modifierNames[0]))
 
+////////////////////////////////////////////////////////////////////////////////
+static inline std::string& str_replace (
+  std::string &str,
+  const std::string& search,
+  const std::string& replacement)
+{
+  std::string::size_type pos = 0;
+  while ((pos = str.find (search, pos)) != std::string::npos)
+  {
+    str.replace (pos, search.length (), replacement);
+    pos += replacement.length ();
+  }
+
+  return str;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 Att::Att ()
 : mName ("")
@@ -186,14 +205,18 @@ bool Att::valid (const std::string& input) const
       if (!n.getUntilOneOf (".:", ignored))
         return false;
 
-    if (n.skip (':') &&
-        (n.getQuoted ('"', ignored) ||
-         n.getUntil  (' ', ignored) ||
-         n.getUntilEOS (ignored)    ||
-         n.depleted ()))
-      return true;
+    if (n.skip (':'))
+    {
+      if (input.find ('@') <= n.cursor () ||
+          input.find ('/') <= n.cursor ())
+        return false;
 
-    return false;
+      if (n.getQuoted ('"', ignored) ||
+          n.getUntil  (' ', ignored) ||
+          n.getUntilEOS (ignored)    ||
+          n.depleted ())
+        return true;
+    }
   }
 
   return false;
@@ -448,7 +471,8 @@ std::string Att::type (const std::string& name) const
   else if (name == "recur")
     return "duration";
 
-  else if (name == "limit")
+  else if (name == "limit" ||
+           name == "urgency")
     return "number";
 
   else if (name == "priority")
@@ -472,10 +496,12 @@ std::string Att::modType (const std::string& name) const
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-//
+//                                  ______________
+//                                  |            |
+//                                  |            v
 // start --> name --> . --> mod --> : --> " --> value --> " --> end
-//            |                     ^
-//            |_____________________|
+//            |                     ^              |             ^
+//            |_____________________|              |_____________|
 //
 void Att::parse (const std::string& input)
 {
@@ -513,8 +539,8 @@ void Att::parse (Nibbler& n)
     {
       // Both quoted and unquoted Att's are accepted.
       // Consider removing this for a stricter parse.
-      if (n.getQuoted ('"', mValue) ||
-          n.getUntil  (' ', mValue))
+      if (n.getQuoted   ('"', mValue) ||
+          n.getUntilEOS (mValue))
       {
         decode (mValue);
       }
@@ -533,11 +559,22 @@ void Att::parse (Nibbler& n)
 ////////////////////////////////////////////////////////////////////////////////
 // "this" is the attribute that has modifiers.  "other" is the attribute from a
 // Record that does not have modifiers, but may have a value.
+//
+// In other words, the filter:
+//   task list description.contains:foo
+//
+// Is represented with:
+//   this  = filter (description.contains:foo)
+//   other = actual task data to be matched
+//
 bool Att::match (const Att& other) const
 {
   // All matches are assumed to pass, any short-circuit on non-match.
   bool case_sensitive = context.config.getBoolean ("search.case.sensitive");
 
+  // Are regular expressions being used in place of string comparison?
+  bool regex = context.config.getBoolean ("regex");
+
   // If there are no mods, just perform a straight compare on value.
   if (mMod == "")
   {
@@ -557,30 +594,73 @@ bool Att::match (const Att& other) const
     }
     else
     {
+#ifdef FEATURE_REGEX
+      if (regex)
+      {
+        std::string pattern = "^" + mValue + "$";
+        if (!regexMatch (other.mValue, pattern, case_sensitive))
+          return false;
+      }
+      else if (!compare (mValue, other.mValue, (bool) case_sensitive))
+        return false;
+#else
       if (!compare (mValue, other.mValue, (bool) case_sensitive))
         return false;
+#endif
     }
   }
 
   // has = contains as a substring.
   else if (mMod == "has" || mMod == "contains") // TODO i18n
   {
+#ifdef FEATURE_REGEX
+    if (regex)
+    {
+      if (!regexMatch (other.mValue, mValue, case_sensitive))
+        return false;
+    }
+    else if (find (other.mValue, mValue, (bool) case_sensitive) == std::string::npos)
+      return false;
+#else
     if (find (other.mValue, mValue, (bool) case_sensitive) == std::string::npos)
       return false;
+#endif
   }
 
   // is = equal.  Nop.
   else if (mMod == "is" || mMod == "equals") // TODO i18n
   {
+#ifdef FEATURE_REGEX
+    if (regex)
+    {
+      std::string pattern = "^" + mValue + "$";
+      if (!regexMatch (other.mValue, pattern, case_sensitive))
+        return false;
+    }
+    else if (!compare (mValue, other.mValue, (bool) case_sensitive))
+      return false;
+#else
     if (!compare (mValue, other.mValue, (bool) case_sensitive))
       return false;
+#endif
   }
 
   // isnt = not equal.
   else if (mMod == "isnt" || mMod == "not") // TODO i18n
   {
+#ifdef FEATURE_REGEX
+    if (regex)
+    {
+      std::string pattern = "^" + mValue + "$";
+      if (regexMatch (other.mValue, pattern, case_sensitive))
+        return false;
+    }
+    else if (compare (mValue, other.mValue, (bool) case_sensitive))
+      return false;
+#else
     if (compare (mValue, other.mValue, (bool) case_sensitive))
       return false;
+#endif
   }
 
   // any = any value, but not empty value.
@@ -600,30 +680,66 @@ bool Att::match (const Att& other) const
   // startswith = first characters must match.
   else if (mMod == "startswith" || mMod == "left") // TODO i18n
   {
-    if (other.mValue.length () < mValue.length ())
-      return false;
+#ifdef FEATURE_REGEX
+    if (regex)
+    {
+      std::string pattern = "^" + mValue;
+      if (!regexMatch (other.mValue, pattern, case_sensitive))
+        return false;
+    }
+    else
+    {
+#endif
+      if (other.mValue.length () < mValue.length ())
+        return false;
 
       if (!compare (mValue, other.mValue.substr (0, mValue.length ()), (bool) case_sensitive))
-      return false;
+        return false;
+#ifdef FEATURE_REGEX
+    }
+#endif
   }
 
   // endswith = last characters must match.
   else if (mMod == "endswith" || mMod == "right") // TODO i18n
   {
-    if (other.mValue.length () < mValue.length ())
-      return false;
+#ifdef FEATURE_REGEX
+    if (regex)
+    {
+      std::string pattern = mValue + "$";
+      if (!regexMatch (other.mValue, pattern, case_sensitive))
+        return false;
+    }
+    else
+    {
+#endif
+      if (other.mValue.length () < mValue.length ())
+        return false;
 
-    if (!compare (mValue, other.mValue.substr (
-                    other.mValue.length () - mValue.length (),
-                    std::string::npos), (bool) case_sensitive))
-      return false;
+      if (!compare (mValue, other.mValue.substr (
+                      other.mValue.length () - mValue.length (),
+                      std::string::npos), (bool) case_sensitive))
+        return false;
+#ifdef FEATURE_REGEX
+    }
+#endif
   }
 
   // hasnt = does not contain as a substring.
   else if (mMod == "hasnt") // TODO i18n
   {
+#ifdef FEATURE_REGEX
+    if (regex)
+    {
+      if (regexMatch (other.mValue, mValue, case_sensitive))
+        return false;
+    }
+    else if (find (other.mValue, mValue, (bool) case_sensitive) != std::string::npos)
+      return false;
+#else
     if (find (other.mValue, mValue, (bool) case_sensitive) != std::string::npos)
       return false;
+#endif
   }
 
   // before = under = below = <
@@ -705,30 +821,66 @@ bool Att::match (const Att& other) const
   // word = contains as a substring, with word boundaries.
   else if (mMod == "word") // TODO i18n
   {
-    // Fail if the substring is not found.
-    std::string::size_type sub = find (other.mValue, mValue, (bool) case_sensitive);
-    if (sub == std::string::npos)
-      return false;
+#ifdef FEATURE_REGEX
+    if (regex && other.mName != "tags")
+    {
+      std::vector <int> start;
+      std::vector <int> end;
+      if (!regexMatch (start, end, other.mValue, mValue, case_sensitive))
+        return false;
 
-    // Also fail if there is no word boundary at beginning and end.
-    if (!isWordStart (other.mValue, sub))
-      return false;
+      if (!isWordStart (other.mValue, start[0]))
+        return false;
 
-    if (!isWordEnd (other.mValue, sub + mValue.length () - 1))
-      return false;
+      if (!isWordEnd (other.mValue, end[0]))
+        return false;
+    }
+    else
+    {
+#endif
+      // Fail if the substring is not found.
+      std::string::size_type sub = find (other.mValue, mValue, (bool) case_sensitive);
+      if (sub == std::string::npos)
+        return false;
+
+      // Also fail if there is no word boundary at beginning and end.
+      if (!isWordStart (other.mValue, sub))
+        return false;
+
+      if (!isWordEnd (other.mValue, sub + mValue.length () - 1))
+        return false;
+#ifdef FEATURE_REGEX
+    }
+#endif
   }
 
   // noword = does not contain as a substring, with word boundaries.
   else if (mMod == "noword") // TODO i18n
   {
-    // Fail if the substring is not found.
-    std::string::size_type sub = find (other.mValue, mValue);
-    if (sub != std::string::npos &&
-        isWordStart (other.mValue, sub) &&
-        isWordEnd (other.mValue, sub + mValue.length () - 1))
+#ifdef FEATURE_REGEX
+    if (regex && other.mName != "tags")
     {
-      return false;
+      std::vector <int> start;
+      std::vector <int> end;
+      if (regexMatch (start, end, other.mValue, mValue, case_sensitive) &&
+          isWordStart (other.mValue, start[0])                     &&
+          isWordEnd (other.mValue, end[0]))
+        return false;
     }
+    else
+    {
+#endif
+      // Fail if the substring is not found.
+      std::string::size_type sub = find (other.mValue, mValue);
+      if (sub != std::string::npos &&
+          isWordStart (other.mValue, sub) &&
+          isWordEnd (other.mValue, sub + mValue.length () - 1))
+      {
+        return false;
+      }
+#ifdef FEATURE_REGEX
+    }
+#endif
   }
 
   return true;
@@ -831,9 +983,7 @@ void Att::enquote (std::string& value) const
 // that hand-editing the pending.data file could cause.
 void Att::dequote (std::string& value) const
 {
-  std::string::size_type quote;
-  while ((quote = value.find ('"')) != std::string::npos)
-    value.replace (quote, 1, "");
+  str_replace (value, "\"", "");
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -842,21 +992,14 @@ void Att::dequote (std::string& value) const
 //   "  -> &dquot;
 //   [  -> &open;
 //   ]  -> &close;
+//   \  -> \\               (extra chars to disambiguate multi-line comment)
 void Att::encode (std::string& value) const
 {
-  std::string::size_type i;
-
-  while ((i = value.find ('\t')) != std::string::npos)
-    value.replace (i, 1, "&tab;"); // no i18n
-
-  while ((i = value.find ('"')) != std::string::npos)
-    value.replace (i, 1, "&dquot;"); // no i18n
-
-  while ((i = value.find ('[')) != std::string::npos)
-    value.replace (i, 1, "&open;"); // no i18n
-
-  while ((i = value.find (']')) != std::string::npos)
-    value.replace (i, 1, "&close;"); // no i18n
+  str_replace (value, "\t", "&tab;");
+  str_replace (value, "\"", "&dquot;");
+  str_replace (value, "[",  "&open;");
+  str_replace (value, "]",  "&close;");
+  str_replace (value, "\\", "\\\\");
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -870,35 +1013,19 @@ void Att::encode (std::string& value) const
 //   :  <- &colon;
 void Att::decode (std::string& value) const
 {
-  std::string::size_type i;
-
   // Supported encodings.
-  while ((i = value.find ("&tab;")) != std::string::npos)
-    value.replace (i, 5, "\t");
-
-  while ((i = value.find ("&dquot;")) != std::string::npos)
-    value.replace (i, 7, "\"");
-
-  while ((i = value.find ("&quot;")) != std::string::npos)
-    value.replace (i, 6, "\"");
-
-  while ((i = value.find ("&open;")) != std::string::npos)
-    value.replace (i, 6, "[");
-
-  while ((i = value.find ("&close;")) != std::string::npos)
-    value.replace (i, 7, "]");
+  str_replace (value, "&tab;",   "\t");
+  str_replace (value, "&dquot;", "\"");
+  str_replace (value, "&quot;",  "'");
+  str_replace (value, "&open;",  "[");
+  str_replace (value, "&close;", "]");
 
   // Support for deprecated encodings.  These cannot be removed or old files
   // will not be parsable.  Not just old files - completed.data can contain
   // tasks formatted/encoded using these.
-  while ((i = value.find ("&squot;")) != std::string::npos)
-    value.replace (i, 7, "'");
-
-  while ((i = value.find ("&comma;")) != std::string::npos)
-    value.replace (i, 7, ",");
-
-  while ((i = value.find ("&colon;")) != std::string::npos)
-    value.replace (i, 7, ":");
+  str_replace (value, "&squot;", "'");
+  str_replace (value, "&comma;", ",");
+  str_replace (value, "&colon;", ":");
 }
 
 ////////////////////////////////////////////////////////////////////////////////
