@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -135,6 +135,9 @@ void Cmd::load ()
     commands.push_back ("_config");
     commands.push_back ("_version");
     commands.push_back ("_urgency");
+    commands.push_back ("_query");
+    commands.push_back ("_zshcommands");
+    commands.push_back ("_zshids");
     commands.push_back ("export.csv");
     commands.push_back ("export.ical");
     commands.push_back ("export.yaml");
@@ -142,6 +145,10 @@ void Cmd::load ()
     commands.push_back ("history.annual");
     commands.push_back ("ghistory.monthly");
     commands.push_back ("ghistory.annual");
+    commands.push_back ("burndown.daily");
+    commands.push_back ("burndown.weekly");
+    commands.push_back ("burndown.monthly");
+    commands.push_back ("count");
 
     // Commands whose names are localized.
     commands.push_back (context.stringtable.get (CMD_ADD,         "add"));
@@ -237,6 +244,9 @@ bool Cmd::isReadOnlyCommand ()
       command == "_config"                                                   ||
       command == "_version"                                                  ||
       command == "_urgency"                                                  ||
+      command == "_query"                                                    ||
+      command == "_zshcommands"                                              ||
+      command == "_zshids"                                                   ||
       command == "export.csv"                                                ||
       command == "export.ical"                                               ||
       command == "export.yaml"                                               ||
@@ -244,6 +254,10 @@ bool Cmd::isReadOnlyCommand ()
       command == "history.annual"                                            ||
       command == "ghistory.monthly"                                          ||
       command == "ghistory.annual"                                           ||
+      command == "burndown.daily"                                            ||
+      command == "burndown.weekly"                                           ||
+      command == "burndown.monthly"                                          ||
+      command == "count"                                                     ||
       command == context.stringtable.get (CMD_CALENDAR,    "calendar")       ||
       command == context.stringtable.get (CMD_COLORS,      "colors")         ||
       command == context.stringtable.get (CMD_DIAGNOSTICS, "diagnostics")    ||
