@@ -1,7 +1,7 @@
 ///////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -48,7 +48,7 @@
 std::string Config::defaults =
   "# Taskwarrior program configuration file.\n"
   "# For more documentation, see http://taskwarrior.org or try 'man task', 'man task-faq',\n"
-  "# 'man task-tutorial', 'man task-color' or 'man taskrc'\n"
+  "# 'man task-tutorial', 'man task-color', 'man task-sync' or 'man taskrc'\n"
   "\n"
   "# Here is an example of entries that use the default, override and blank values\n"
   "#   variable=foo   -- By specifying a value, this overrides the default\n"
@@ -60,11 +60,13 @@ std::string Config::defaults =
   "# Files\n"
   "data.location=~/.task\n"
   "locking=on                                     # Use file-level locking\n"
+  "gc=on                                          # Garbage-collect data files - DO NOT CHANGE unless you are sure\n"
   "\n"
   "# Terminal\n"
-  "curses=on                                      # Use ncurses library to determine terminal width\n"
-  "defaultwidth=80                                # Without ncurses, assumed width\n"
+  "curses=on                                      # Detects terminal width\n"
+  "defaultwidth=80                                # Without detection, assumed width\n"
   "#editor=vi                                     # Preferred text editor\n"
+  "edit.verbose=yes                               # Include comments in files created during task edit\n"
   "\n"
   "# Miscellaneous\n"
   "verbose=yes                                    # Provide extra feedback\n"
@@ -80,6 +82,9 @@ std::string Config::defaults =
   "recurrence.indicator=R                         # What to show as a task recurrence indicator\n"
   "recurrence.limit=1                             # Number of future recurring pending tasks\n"
   "undo.style=side                                # Undo style - can be 'side', or 'diff'\n"
+  "burndown.bias=0.666                            # Weighted mean bias toward recent data\n"
+  "regex=no                                       # Assume all search/filter strings are regexes\n"
+  "xterm.title=no                                 # Sets xterm title for some commands\n"
   "\n"
   "# Dates\n"
   "dateformat=m/d/Y                               # Preferred input and display date format\n"
@@ -94,6 +99,8 @@ std::string Config::defaults =
   "calendar.legend=yes                            # Display the legend on calendar\n"
   "calendar.details=sparse                        # Calendar shows information for tasks w/due dates: full, sparse or none\n"
   "calendar.details.report=list                   # Report to use when showing task information in cal\n"
+  "calendar.offset=no                             # Apply an offset value to control the first month of the calendar\n"
+  "calendar.offset.value=-1                       # The number of months the first month of the calendar is moved\n"
   "calendar.holidays=none                         # Show public holidays on calendar:full, sparse or none\n"
   "#monthsperline=3                               # Number of calendar months on a line\n"
   "\n"
@@ -101,6 +108,7 @@ std::string Config::defaults =
   "journal.time=no                                # Record start/stop commands as annotation\n"
   "journal.time.start.annotation=Started task     # Annotation description for the start journal entry\n"
   "journal.time.stop.annotation=Stopped task      # Annotation description for the stop  journal entry\n"
+  "journal.info=on                                # Display task journal with info command\n"
   "\n"
   "# Dependency controls\n"
   "dependency.reminder=on                         # Nags on dependency chain violations\n"
@@ -129,13 +137,17 @@ std::string Config::defaults =
   "color.debug=color3                             # Color of diagnostic output\n"
   "color.alternate=on color233                    # Alternate color for line coloring\n"
   "\n"
-  "color.summary.bar=on rgb141                    # Color of summary report progress bar\n"
-  "color.summary.background=on color0             # Color of summary report background\n"
+  "color.summary.bar=black on rgb141              # Color of summary report progress bar\n"
+  "color.summary.background=white on color0       # Color of summary report background\n"
   "\n"
   "color.history.add=color0 on rgb500             # Color of added tasks in ghistory report\n"
   "color.history.done=color0 on rgb050            # Color of completed tasks in ghistory report\n"
   "color.history.delete=color0 on rgb550          # Color of deleted tasks in ghistory report\n"
   "\n"
+  "color.burndown.pending=color0 on rgb500        # Color of pending tasks in burndown report\n"
+  "color.burndown.done=color0 on rgb050           # Color of completed tasks in burndown report\n"
+  "color.burndown.started=color0 on rgb550        # Color of started tasks in burndown report\n"
+  "\n"
   "color.sync.added=rgb005                        # Color of added tasks in sync output\n"
   "color.sync.changed=rgb550                      # Color of changed tasks in sync output\n"
   "color.sync.rejected=rgb500                     # Color of rejected tasks in sync output\n"
@@ -174,13 +186,17 @@ std::string Config::defaults =
   "color.debug=yellow                             # Color of diagnostic output\n"
   "color.alternate=                               # Alternate color for line coloring\n"
   "\n"
-  "color.summary.bar=on green                     # Color of summary report progress bar\n"
-  "color.summary.background=on black              # Color of summary report background\n"
+  "color.summary.bar=black on green               # Color of summary report progress bar\n"
+  "color.summary.background=white on black        # Color of summary report background\n"
   "\n"
   "color.history.add=black on red                 # Color of added tasks in ghistory report\n"
   "color.history.done=black on green              # Color of completed tasks in ghistory report\n"
   "color.history.delete=black on yellow           # Color of deleted tasks in ghistory report\n"
   "\n"
+  "color.burndown.pending=black on red            # Color of pending tasks in burndown report\n"
+  "color.burndown.done=black on green             # Color of completed tasks in burndown report\n"
+  "color.burndown.started=black on yellow         # Color of started tasks in burndown report\n"
+  "\n"
   "color.sync.added=green                         # Color of added tasks in sync output\n"
   "color.sync.changed=yellow                      # Color of changed tasks in sync output\n"
   "color.sync.rejected=red                        # Color of rejected tasks in sync output\n"
@@ -227,6 +243,7 @@ std::string Config::defaults =
   "\n"
   "#default.project=foo                           # Default project for 'add' command\n"
   "#default.priority=M                            # Default priority for 'add' command\n"
+  "#default.due=eom                               # Default due date for 'add' command\n"
   "default.command=list                           # When no arguments are specified\n"
   "\n"
   "_forcecolor=no                                 # Forces color to be on, even for non TTY output\n"
@@ -273,6 +290,8 @@ std::string Config::defaults =
   "alias.ghistory=ghistory.monthly                # Prefer monthly graphical over annual history reports\n"
   "alias.export=export.yaml                       # Prefer YAML over CSV or iCal export\n"
   "alias.export.vcalendar=export.ical             # They are the same\n"
+  "alias.export.json=_query                       # The _query command will be used for all export\n"
+  "alias.burndown=burndown.weekly                 # Prefer the weekly burndown chart\n"
   "\n"
   "# Fields: id, uuid, project, priority, priority_long, entry, start, end,\n"
   "#         due, countdown, countdown_compact, age, age_compact, active, tags,\n"
@@ -287,7 +306,7 @@ std::string Config::defaults =
   "report.long.description=Lists all task, all data, matching the specified criteria\n"
   "report.long.columns=id,project,priority,entry,start,due,recur,countdown,age,depends,tags,description\n"
   "report.long.labels=ID,Project,Pri,Added,Started,Due,Recur,Countdown,Age,Deps,Tags,Description\n"
-  "report.long.sort=due+,priority-,project+\n"
+  "report.long.sort=due+,priority-,project+,description+\n"
   "report.long.filter=status:pending\n"
   "#report.long.dateformat=m/d/Y\n"
   "#report.long.annotations=full\n"
@@ -296,7 +315,7 @@ std::string Config::defaults =
   "report.list.description=Lists all tasks matching the specified criteria\n"
   "report.list.columns=id,project,priority,due,active,age,description\n"
   "report.list.labels=ID,Project,Pri,Due,Active,Age,Description\n"
-  "report.list.sort=due+,priority-,active-,project+\n"
+  "report.list.sort=due+,priority-,active-,project+,description+\n"
   "report.list.filter=status:pending\n"
   "#report.list.dateformat=m/d/Y\n"
   "#report.list.annotations=full\n"
@@ -305,7 +324,7 @@ std::string Config::defaults =
   "report.ls.description=Minimal listing of all tasks matching the specified criteria\n"
   "report.ls.columns=id,project,priority,description\n"
   "report.ls.labels=ID,Project,Pri,Description\n"
-  "report.ls.sort=priority-,project+\n"
+  "report.ls.sort=priority-,project+,description+\n"
   "report.ls.filter=status:pending\n"
   "#report.ls.dateformat=m/d/Y\n"
   "#report.ls.annotations=full\n"
@@ -341,7 +360,7 @@ std::string Config::defaults =
   "report.overdue.description=Lists overdue tasks matching the specified criteria\n"
   "report.overdue.columns=id,project,priority,due,active,age,description\n"
   "report.overdue.labels=ID,Project,Pri,Due,Active,Age,Description\n"
-  "report.overdue.sort=due+,priority-,active-,project+\n"
+  "report.overdue.sort=due+,priority-,active-,project+,description+\n"
   "report.overdue.filter=status:pending due.before:now\n"
   "#report.overdue.dateformat=m/d/Y\n"
   "#report.overdue.annotations=full\n"
@@ -350,7 +369,7 @@ std::string Config::defaults =
   "report.active.description=Lists active tasks matching the specified criteria\n"
   "report.active.columns=id,project,priority,due,active,age,description\n"
   "report.active.labels=ID,Project,Pri,Due,Active,Age,Description\n"
-  "report.active.sort=due+,priority-,project+\n"
+  "report.active.sort=due+,priority-,project+,description+\n"
   "report.active.filter=status:pending start.any:\n"
   "#report.active.dateformat=m/d/Y\n"
   "#report.active.annotations=full\n"
@@ -359,7 +378,7 @@ std::string Config::defaults =
   "report.completed.description=Lists completed tasks matching the specified criteria\n"
   "report.completed.columns=end,project,priority,age,description\n"
   "report.completed.labels=Complete,Project,Pri,Age,Description\n"
-  "report.completed.sort=end+,priority-,project+\n"
+  "report.completed.sort=end+,priority-,project+,description+\n"
   "report.completed.filter=status:completed\n"
   "#report.completed.dateformat=m/d/Y\n"
   "#report.completed.annotations=full\n"
@@ -368,7 +387,7 @@ std::string Config::defaults =
   "report.recurring.description=Lists recurring tasks matching the specified criteria\n"
   "report.recurring.columns=id,project,priority,due,recur,active,age,description\n"
   "report.recurring.labels=ID,Project,Pri,Due,Recur,Active,Age,Description\n"
-  "report.recurring.sort=due+,priority-,active-,project+\n"
+  "report.recurring.sort=due+,priority-,active-,project+,description+\n"
   "report.recurring.filter=status:pending parent.any:\n"
   "#report.recurring.dateformat=m/d/Y\n"
   "#report.recurring.annotations=full\n"
@@ -377,13 +396,13 @@ std::string Config::defaults =
   "report.waiting.description=Lists all waiting tasks matching the specified criteria\n"
   "report.waiting.columns=id,project,priority,wait,age,description\n"
   "report.waiting.labels=ID,Project,Pri,Wait,Age,Description\n"
-  "report.waiting.sort=wait+,priority-,project+\n"
+  "report.waiting.sort=wait+,priority-,project+,description+\n"
   "report.waiting.filter=status:waiting\n"
   "#report.waiting.dateformat=m/d/Y\n"
   "#report.waiting.annotations=full\n"
   "\n"
   "# task all\n"
-  "report.all.description=Lists all tasks matching the specified criteria\n"
+  "report.all.description=Lists all tasks matching the specified criteria, including parents of recurring tasks\n"
   "report.all.columns=id,project,priority,due,end,active,age,description\n"
   "report.all.labels=ID,Project,Pri,Due,Completed,Active,Age,Description\n"
   "report.all.sort=project+,due+,end+,priority-,active-,description+\n"
@@ -395,7 +414,7 @@ std::string Config::defaults =
   "report.next.description=Lists the most urgent tasks\n"
   "report.next.columns=id,project,priority,due,active,age,description\n"
   "report.next.labels=ID,Project,Pri,Due,Active,Age,Description\n"
-  "report.next.sort=due+,priority-,active-,project+\n"
+  "report.next.sort=due+,priority-,active-,project+,description+\n"
   "report.next.filter=status:pending limit:page depends.none:\n"
   "#report.next.dateformat=m/d/Y\n"
   "#report.next.annotations=full\n"
@@ -404,7 +423,7 @@ std::string Config::defaults =
   "report.blocked.description=Lists all blocked tasks matching the specified criteria\n"
   "report.blocked.columns=id,depends,project,priority,due,active,age,description\n"
   "report.blocked.labels=ID,Deps,Project,Pri,Due,Active,Age,Description\n"
-  "report.blocked.sort=due+,priority-,active-,project+\n"
+  "report.blocked.sort=due+,priority-,active-,project+,description+\n"
   "report.blocked.filter=status:pending depends.any:\n"
   "#report.blocked.dateformat=m/d/Y\n"
   "\n"
@@ -412,7 +431,7 @@ std::string Config::defaults =
   "report.unblocked.description=Lists all unblocked tasks matching the specified criteria\n"
   "report.unblocked.columns=id,depends,project,priority,due,active,age,description\n"
   "report.unblocked.labels=ID,Deps,Project,Pri,Due,Active,Age,Description\n"
-  "report.unblocked.sort=due+,priority-,active-,project+\n"
+  "report.unblocked.sort=due+,priority-,active-,project+,description+\n"
   "report.unblocked.filter=status:pending depends.none:\n"
   "#report.unblocked.dateformat=m/d/Y\n"
   "\n";
@@ -624,17 +643,13 @@ const bool Config::getBoolean (const std::string& key)
 ////////////////////////////////////////////////////////////////////////////////
 void Config::set (const std::string& key, const int value)
 {
-  char v[24];
-  sprintf (v, "%d", value);
-  (*this)[key] = v;
+  (*this)[key] = format (value);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 void Config::set (const std::string& key, const double value)
 {
-  char v[32];
-  sprintf (v, "%f", value);
-  (*this)[key] = v;
+  (*this)[key] = format (value, 1, 8);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -645,23 +660,25 @@ void Config::set (const std::string& key, const std::string& value)
 
 ////////////////////////////////////////////////////////////////////////////////
 // Provide a vector of all configuration keys.
-void Config::all (std::vector<std::string>& items)
+void Config::all (std::vector<std::string>& items) const
 {
-  foreach (i, *this)
-    items.push_back (i->first);
+  std::map <std::string, std::string>::const_iterator it;
+  for (it = this->begin (); it != this->end (); ++it)
+    items.push_back (it->first);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 std::string Config::checkForDeprecatedColor ()
 {
   std::vector <std::string> deprecated;
-  foreach (i, *this)
+  std::map <std::string, std::string>::const_iterator it;
+  for (it = this->begin (); it != this->end (); ++it)
   {
-    if (i->first.find ("color.") != std::string::npos)
+    if (it->first.find ("color.") != std::string::npos)
     {
-      std::string value = get (i->first);
+      std::string value = get (it->first);
       if (value.find ("_") != std::string::npos)
-        deprecated.push_back (i->first);
+        deprecated.push_back (it->first);
     }
   }
 
@@ -671,8 +688,9 @@ std::string Config::checkForDeprecatedColor ()
     out << "Your .taskrc file contains color settings that use deprecated "
         << "underscores.  Please check:\n";
 
-    foreach (i, deprecated)
-      out << "  " << *i << "=" << get (*i) << "\n";
+    std::vector <std::string>::iterator it2;
+    for (it2 = deprecated.begin (); it2 != deprecated.end (); ++it2)
+      out << "  " << *it2 << "=" << get (*it2) << "\n";
 
     out << "\n";
   }
@@ -684,15 +702,16 @@ std::string Config::checkForDeprecatedColor ()
 std::string Config::checkForDeprecatedColumns ()
 {
   std::vector <std::string> deprecated;
-  foreach (i, *this)
+  std::map <std::string, std::string>::const_iterator it;
+  for (it = this->begin (); it != this->end (); ++it)
   {
-    if (i->first.find ("report") == 0)
+    if (it->first.find ("report") == 0)
     {
-      std::string value = get (i->first);
+      std::string value = get (it->first);
       if (value.find ("entry_time") != std::string::npos ||
           value.find ("start_time") != std::string::npos ||
           value.find ("end_time")   != std::string::npos)
-        deprecated.push_back (i->first);
+        deprecated.push_back (it->first);
     }
   }
 
@@ -704,8 +723,9 @@ std::string Config::checkForDeprecatedColumns ()
     out << "Your .taskrc file contains reports with deprecated columns.  "
         << "Please check for entry_time, start_time or end_time in:\n";
 
-    foreach (i, deprecated)
-      out << "  " << *i << "\n";
+    std::vector <std::string>::iterator it2;
+    for (it2 = deprecated.begin (); it2 != deprecated.end (); ++it2)
+      out << "  " << *it2 << "=" << get (*it2) << "\n";
 
     out << "\n";
   }
