@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -57,7 +57,7 @@ public:
   void set (const std::string&, const int);
   void set (const std::string&, const double);
   void set (const std::string&, const std::string&);
-  void all (std::vector <std::string>&);
+  void all (std::vector <std::string>&) const;
 
   std::string checkForDeprecatedColor ();
   std::string checkForDeprecatedColumns ();
