@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -56,6 +56,8 @@ Context::Context ()
 , var_overrides ("")
 , cmd ()
 , inShadow (false)
+, terminal_width (0)
+, terminal_height (0)
 {
 }
 
@@ -204,6 +206,17 @@ int Context::dispatch (std::string &out)
 
   hooks.trigger ("pre-dispatch");
 
+  // For read-only commands, optionally update the xterm window title.
+  // Why just the read-only commands?  Because this capability is to answer the
+  // question of 'what did I just do to generate this outout?'.
+  if (config.getBoolean ("xterm.title") &&
+      cmd.isReadOnlyCommand ())
+  {
+    std::string title;
+    join (title, " ", args);
+    std::cout << "]0;task " << title << "" << std::endl;
+  }
+
   // TODO Just look at this thing.  It cries out for a dispatch table.
        if (cmd.command == "projects")         { rc = handleProjects              (out); }
   else if (cmd.command == "tags")             { rc = handleTags                  (out); }
@@ -218,6 +231,9 @@ int Context::dispatch (std::string &out)
   else if (cmd.command == "history.annual")   { rc = handleReportHistoryAnnual   (out); }
   else if (cmd.command == "ghistory.monthly") { rc = handleReportGHistoryMonthly (out); }
   else if (cmd.command == "ghistory.annual")  { rc = handleReportGHistoryAnnual  (out); }
+  else if (cmd.command == "burndown.daily")   { rc = handleReportBurndownDaily   (out); }
+  else if (cmd.command == "burndown.weekly")  { rc = handleReportBurndownWeekly  (out); }
+  else if (cmd.command == "burndown.monthly") { rc = handleReportBurndownMonthly (out); }
   else if (cmd.command == "summary")          { rc = handleReportSummary         (out); }
   else if (cmd.command == "calendar")         { rc = handleReportCalendar        (out); }
   else if (cmd.command == "timesheet")        { rc = handleReportTimesheet       (out); }
@@ -246,6 +262,7 @@ int Context::dispatch (std::string &out)
   else if (cmd.command == "push")             {      handlePush                  (out); }
   else if (cmd.command == "pull")             {      handlePull                  (out); }
   else if (cmd.command == "diagnostics")      {      handleDiagnostics           (out); }
+  else if (cmd.command == "count")            { rc = handleCount                 (out); }
   else if (cmd.command == "_projects")        { rc = handleCompletionProjects    (out); }
   else if (cmd.command == "_tags")            { rc = handleCompletionTags        (out); }
   else if (cmd.command == "_commands")        { rc = handleCompletionCommands    (out); }
@@ -253,6 +270,9 @@ int Context::dispatch (std::string &out)
   else if (cmd.command == "_config")          { rc = handleCompletionConfig      (out); }
   else if (cmd.command == "_version")         { rc = handleCompletionVersion     (out); }
   else if (cmd.command == "_urgency")         { rc = handleUrgency               (out); }
+  else if (cmd.command == "_query")           { rc = handleQuery                 (out); }
+  else if (cmd.command == "_zshcommands")     { rc = handleZshCompletionCommands (out); }
+  else if (cmd.command == "_zshids")          { rc = handleZshCompletionIDs      (out); }
   else if (cmd.command == "" &&
            sequence.size ())                  { rc = handleModify                (out); }
 
@@ -292,6 +312,10 @@ void Context::shadow ()
       throw std::string ("Configuration variable 'shadow.file' is set to "
                          "overwrite your completed tasks.  Please change it.");
 
+    if (shadowFile.data == dataLocation + "/undo.data")
+      throw std::string ("Configuration variable 'shadow.file' is set to "
+                         "overwrite your undo log.  Please change it.");
+
     std::string oldCurses = config.get ("curses");
     std::string oldColor  = config.get ("color");
 
@@ -829,6 +853,21 @@ void Context::autoFilter (Att& a, Filter& f)
     }
   }
 
+  // Recurrence periods are matched left-most.
+  else if (a.name () == "recur" && a.mod () == "")
+  {
+    if (a.value () != "")
+    {
+      f.push_back (Att ("recur", "startswith", a.value ()));
+      debug ("auto filter: " + a.name () + ".startswith:" + a.value ());
+    }
+    else
+    {
+      f.push_back (Att ("recur", "is", a.value ()));
+      debug ("auto filter: " + a.name () + ".is:" + a.value ());
+    }
+  }
+
   // The limit attribute does not participate in filtering, and needs to be
   // specifically handled in handleCustomReport.
   else if (a.name () == "limit")
