@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -99,6 +99,9 @@ public:
   std::vector <std::string> footnotes;
   std::vector <std::string> debugMessages;
   bool                      inShadow;
+
+  int                       terminal_width;
+  int                       terminal_height;
 };
 
 #endif
