@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -113,7 +113,7 @@ Date::Date (const std::string& input, const std::string& format /* = "m/d/Y" */)
       if (i >= input.length () ||
           ! isdigit (input[i]))
       {
-        throw std::string ("\"") + input + "\" is not a valid date (m).";
+        throw std::string ("\"") + input + "\" is not a valid date (m).  Expected format '" + format + "'";
       }
 
       if (i + 1 < input.length ()                    &&
@@ -134,7 +134,7 @@ Date::Date (const std::string& input, const std::string& format /* = "m/d/Y" */)
       if (i >= input.length () ||
           ! isdigit (input[i]))
       {
-        throw std::string ("\"") + input + "\" is not a valid date (d).";
+        throw std::string ("\"") + input + "\" is not a valid date (d).  Expected format '" + format + "'";
       }
 
       if (i + 1 < input.length ()                                                                    &&
@@ -157,7 +157,7 @@ Date::Date (const std::string& input, const std::string& format /* = "m/d/Y" */)
           ! isdigit (input[i + 0]) ||
           ! isdigit (input[i + 1]))
       {
-        throw std::string ("\"") + input + "\" is not a valid date (y).";
+        throw std::string ("\"") + input + "\" is not a valid date (y).  Expected format '" + format + "'";
       }
 
       year = atoi (input.substr (i, 2).c_str ()) + 2000;
@@ -169,7 +169,7 @@ Date::Date (const std::string& input, const std::string& format /* = "m/d/Y" */)
           ! isdigit (input[i + 0]) ||
           ! isdigit (input[i + 1]))
       {
-        throw std::string ("\"") + input + "\" is not a valid date (M).";
+        throw std::string ("\"") + input + "\" is not a valid date (M).  Expected format '" + format + "'";
       }
 
       month = atoi (input.substr (i, 2).c_str ());
@@ -181,7 +181,7 @@ Date::Date (const std::string& input, const std::string& format /* = "m/d/Y" */)
           ! isdigit (input[i + 0]) ||
           ! isdigit (input[i + 1]))
       {
-        throw std::string ("\"") + input + "\" is not a valid date (D).";
+        throw std::string ("\"") + input + "\" is not a valid date (D).  Expected format '" + format + "'";
       }
 
       day = atoi (input.substr (i, 2).c_str ());
@@ -193,7 +193,7 @@ Date::Date (const std::string& input, const std::string& format /* = "m/d/Y" */)
           ! isdigit (input[i + 0]) ||
           ! isdigit (input[i + 1]))
       {
-        throw std::string ("\"") + input + "\" is not a valid date (V).";
+        throw std::string ("\"") + input + "\" is not a valid date (V).  Expected format '" + format + "'";
       }
 
       i += 2;
@@ -207,7 +207,7 @@ Date::Date (const std::string& input, const std::string& format /* = "m/d/Y" */)
           ! isdigit (input[i + 2]) ||
           ! isdigit (input[i + 3]))
       {
-        throw std::string ("\"") + input + "\" is not a valid date (Y).";
+        throw std::string ("\"") + input + "\" is not a valid date (Y).  Expected format '" + format + "'";
       }
 
       year = atoi (input.substr (i, 4).c_str ());
@@ -221,7 +221,7 @@ Date::Date (const std::string& input, const std::string& format /* = "m/d/Y" */)
           isdigit (input[i + 1]) ||
           isdigit (input[i + 2]))
       {
-        throw std::string ("\"") + input + "\" is not a valid date (a).";
+        throw std::string ("\"") + input + "\" is not a valid date (a).  Expected format '" + format + "'";
       }
 
       i += 3;
@@ -233,7 +233,7 @@ Date::Date (const std::string& input, const std::string& format /* = "m/d/Y" */)
           isdigit (input[i + 1]) ||
           isdigit (input[i + 2]))
       {
-        throw std::string ("\"") + input + "\" is not a valid date (b).";
+        throw std::string ("\"") + input + "\" is not a valid date (b).  Expected format '" + format + "'";
       }
 
       month = Date::monthOfYear (input.substr (i, 3).c_str());
@@ -247,7 +247,7 @@ Date::Date (const std::string& input, const std::string& format /* = "m/d/Y" */)
           isdigit (input[i + 1]) ||
           isdigit (input[i + 2]))
       {
-        throw std::string ("\"") + input + "\" is not a valid date (A).";
+        throw std::string ("\"") + input + "\" is not a valid date (A).  Expected format '" + format + "'";
       }
 
       i += Date::dayName( Date::dayOfWeek (input.substr (i, 3).c_str()) ).size();
@@ -259,7 +259,7 @@ Date::Date (const std::string& input, const std::string& format /* = "m/d/Y" */)
           isdigit (input[i + 1]) ||
           isdigit (input[i + 2]))
       {
-        throw std::string ("\"") + input + "\" is not a valid date (B).";
+        throw std::string ("\"") + input + "\" is not a valid date (B).  Expected format '" + format + "'";
       }
 
       month = Date::monthOfYear (input.substr (i, 3).c_str());
@@ -271,7 +271,7 @@ Date::Date (const std::string& input, const std::string& format /* = "m/d/Y" */)
       if (i >= input.length () ||
           ! isdigit (input[i]))
       {
-        throw std::string ("\"") + input + "\" is not a valid date (h).";
+        throw std::string ("\"") + input + "\" is not a valid date (h).  Expected format '" + format + "'";
       }
 
       if (i + 1 < input.length ()                    &&
@@ -293,7 +293,7 @@ Date::Date (const std::string& input, const std::string& format /* = "m/d/Y" */)
           ! isdigit (input[i + 0]) ||
           ! isdigit (input[i + 1]))
       {
-        throw std::string ("\"") + input + "\" is not a valid date (H).";
+        throw std::string ("\"") + input + "\" is not a valid date (H).  Expected format '" + format + "'";
       }
 
       hour = atoi (input.substr (i, 2).c_str ());
@@ -305,7 +305,7 @@ Date::Date (const std::string& input, const std::string& format /* = "m/d/Y" */)
           ! isdigit (input[i + 0]) ||
           ! isdigit (input[i + 1]))
       {
-        throw std::string ("\"") + input + "\" is not a valid date (N).";
+        throw std::string ("\"") + input + "\" is not a valid date (N).  Expected format '" + format + "'";
       }
 
       minute = atoi (input.substr (i, 2).c_str ());
@@ -317,7 +317,7 @@ Date::Date (const std::string& input, const std::string& format /* = "m/d/Y" */)
           ! isdigit (input[i + 0]) ||
           ! isdigit (input[i + 1]))
       {
-        throw std::string ("\"") + input + "\" is not a valid date (S).";
+        throw std::string ("\"") + input + "\" is not a valid date (S).  Expected format '" + format + "'";
       }
 
       second = atoi (input.substr (i, 2).c_str ());
@@ -328,7 +328,7 @@ Date::Date (const std::string& input, const std::string& format /* = "m/d/Y" */)
       if (i >= input.length () ||
           input[i] != format[f])
       {
-        throw std::string ("\"") + input + "\" is not a valid date (DEFAULT).";
+        throw std::string ("\"") + input + "\" is not a valid date.  Expected format '" + format + "'";
       }
       ++i;
       break;
@@ -344,10 +344,10 @@ Date::Date (const std::string& input, const std::string& format /* = "m/d/Y" */)
   }
 
   if (i < input.length ())
-    throw std::string ("\"") + input + "\" is not a valid date in " + format + " format.";
+    throw std::string ("\"") + input + "\" is not a valid date.  Expected format '" + format + "'";
 
   if (!valid (month, day, year))
-    throw std::string ("\"") + input + "\" is not a valid date (VALID).";
+    throw std::string ("\"") + input + "\" is not a valid date but is in a valid format.";
 
   // Convert to epoch.
   struct tm t = {0};
@@ -460,6 +460,32 @@ const std::string Date::toString (const std::string& format /*= "m/d/Y" */) cons
   return formatted;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+Date Date::startOfDay () const
+{
+  return Date (month (), day (), year (), 0, 0, 0);
+}
+
+////////////////////////////////////////////////////////////////////////////////
+Date Date::startOfWeek () const
+{
+  Date sow (mT);
+  sow -= (dayOfWeek () * 86400);
+  return Date (sow.month (), sow.day (), sow.year (), 0, 0, 0);
+}
+
+////////////////////////////////////////////////////////////////////////////////
+Date Date::startOfMonth () const
+{
+  return Date (month (), 1, year (), 0, 0, 0);
+}
+
+////////////////////////////////////////////////////////////////////////////////
+Date Date::startOfYear () const
+{
+  return Date (1, 1, year (), 0, 0, 0);
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 bool Date::valid (const std::string& input, const std::string& format)
 {
@@ -839,6 +865,62 @@ time_t Date::operator- (const Date& rhs)
   return mT - rhs.mT;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+// Prefix decrement by one day.
+void Date::operator-- ()
+{
+  Date yesterday = startOfDay () - 1;
+  yesterday = Date (yesterday.month (),
+                    yesterday.day (),
+                    yesterday.year (),
+                    hour (),
+                    minute (),
+                    second ());
+  mT = yesterday.mT;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// Postfix decrement by one day.
+void Date::operator-- (int)
+{
+  Date yesterday = startOfDay () - 1;
+  yesterday = Date (yesterday.month (),
+                    yesterday.day (),
+                    yesterday.year (),
+                    hour (),
+                    minute (),
+                    second ());
+  mT = yesterday.mT;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// Prefix increment by one day.
+void Date::operator++ ()
+{
+  Date tomorrow = (startOfDay () + 90001).startOfDay ();
+  tomorrow = Date (tomorrow.month (),
+                   tomorrow.day (),
+                   tomorrow.year (),
+                   hour (),
+                   minute (),
+                   second ());
+  mT = tomorrow.mT;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// Postfix increment by one day.
+void Date::operator++ (int)
+{
+  Date tomorrow = (startOfDay () + 90001).startOfDay ();
+  tomorrow = Date (tomorrow.month (),
+                   tomorrow.day (),
+                   tomorrow.year (),
+                   hour (),
+                   minute (),
+                   second ());
+  mT = tomorrow.mT;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 bool Date::isEpoch (const std::string& input)
 {
@@ -903,6 +985,8 @@ bool Date::isRelativeDate (const std::string& input)
   supported.push_back ("midsommar");
   supported.push_back ("midsommarafton");
   supported.push_back ("now");
+  supported.push_back ("later");
+  supported.push_back ("someday");
 
   std::vector <std::string> matches;
   if (autoComplete (in, supported, matches) == 1)
@@ -1059,6 +1143,12 @@ bool Date::isRelativeDate (const std::string& input)
       mT = time (NULL);
       return true;
     }
+    else if (found == "later" || found == "someday")
+    {
+      Date then (1, 18, 2038);
+      mT = then.mT;
+      return true;
+    }
   }
 
   // Support "21st" to indicate the next date that is the 21st day.
