@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -50,6 +50,12 @@ public:
   std::string toISO ();
   void toMDY (int&, int&, int&);
   const std::string toString (const std::string& format = "m/d/Y") const;
+
+  Date startOfDay () const;
+  Date startOfWeek () const;
+  Date startOfMonth () const;
+  Date startOfYear () const;
+
   static bool valid (const std::string&, const std::string& format = "m/d/Y");
   static bool valid (const int, const int, const int, const int, const int, const int);
   static bool valid (const int, const int, const int);
@@ -91,6 +97,11 @@ public:
 
   time_t operator- (const Date&);
 
+  void operator-- ();    // Prefix
+  void operator-- (int); // Postfix
+  void operator++ ();    // Prefix
+  void operator++ (int); // Postfix
+
 private:
   bool isEpoch (const std::string&);
   bool isRelativeDate (const std::string&);
