@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -147,6 +147,8 @@ Hooks::Hooks ()
   validProgramEvents.push_back ("post-done-command");
   validProgramEvents.push_back ("pre-duplicate-command");
   validProgramEvents.push_back ("post-duplicate-command");
+  validProgramEvents.push_back ("pre-count-command");
+  validProgramEvents.push_back ("post-count-command");
   validProgramEvents.push_back ("pre-edit-command");
   validProgramEvents.push_back ("post-edit-command");
   validProgramEvents.push_back ("pre-export-command");
@@ -155,6 +157,8 @@ Hooks::Hooks ()
   validProgramEvents.push_back ("post-ghistory-command");
   validProgramEvents.push_back ("pre-history-command");
   validProgramEvents.push_back ("post-history-command");
+  validProgramEvents.push_back ("pre-burndown-command");
+  validProgramEvents.push_back ("post-burndown-command");
   validProgramEvents.push_back ("pre-import-command");
   validProgramEvents.push_back ("post-import-command");
   validProgramEvents.push_back ("pre-info-command");
