@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -30,7 +30,7 @@
 #include <vector>
 #include <string>
 #include "API.h"
-#include "auto.h"
+#include "../auto.h"
 
 // Hook class representing a single hook, which is just a three-way map.
 class Hook
