@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -28,8 +28,9 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
-#include "Nibbler.h"
-#include "rx.h"
+#include <inttypes.h>
+#include <Nibbler.h>
+#include <rx.h>
 
 const char* c_digits = "0123456789";
 
@@ -213,66 +214,51 @@ bool Nibbler::getUntilEOS (std::string& result)
 bool Nibbler::getQuoted (
   char c,
   std::string& result,
-  bool unescape /* = true */,
   bool quote /* = false */)
 {
-  if (mCursor < mLength)
+  bool inquote = false;
+  bool inescape = false;
+  char current = 0;
+  char previous = 0;
+  result = "";
+
+  if (mCursor >= mLength ||
+      mInput[mCursor] != c)
   {
-    if (mInput[mCursor] != c)
-      return false;
+    return false;
+  }
 
-    result = "";
-    bool inquote = false;
-    char current = 0;
-    char previous = 0;
+  for (std::string::size_type i = mCursor; i < mLength; ++i)
+  {
+    previous = current;
+    current = mInput[i];
 
-    // '"'
-    // p c
-    // - -
-    //   '
-    // ' "
-    // " '
-    for (std::string::size_type i = mCursor; i < mLength; ++i)
+    if (current == '\\' && !inescape)
     {
-      previous = current;
-      current = mInput[i];
+      inescape = true;
+      continue;
+    }
 
-      if (current == c)
+    if (current == c && !inescape)
+    {
+      if (quote)
+        result += current;
+
+      if (!inquote)
       {
-        if (previous == '\\')
-        {
-           if (!unescape)
-             result += previous;
-
-           result += current;
-        }
-        else
-        {
-          if (!inquote)
-          {
-            inquote = true;
-            if (quote)
-              result += current;
-          }
-          else
-          {
-            if (quote)
-              result += current;
-
-            mCursor = i + 1;
-            return true;
-          }
-        }
-      }
-      else if (current == '\\')
-      {
-        // NOP
+        inquote = true;
       }
       else
       {
-        result += current;
+        mCursor = i + 1;
+        return true;
       }
     }
+    else
+    {
+      result += current;
+      inescape = false;
+    }
   }
 
   return false;
@@ -297,7 +283,34 @@ bool Nibbler::getInt (int& result)
 
   if (i > mCursor)
   {
-    result = atoi (mInput.substr (mCursor, i - mCursor).c_str ());
+    result = strtoimax (mInput.substr (mCursor, i - mCursor).c_str (), NULL, 10);
+    mCursor = i;
+    return true;
+  }
+
+  return false;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+bool Nibbler::getHex (int& result)
+{
+  std::string::size_type i = mCursor;
+
+  if (i < mLength)
+  {
+    if (mInput[i] == '-')
+      ++i;
+    else if (mInput[i] == '+')
+      ++i;
+  }
+
+  // TODO Potential for use of find_first_not_of
+  while (i < mLength && isxdigit (mInput[i]))
+    ++i;
+
+  if (i > mCursor)
+  {
+    result = strtoimax (mInput.substr (mCursor, i - mCursor).c_str (), NULL, 16);
     mCursor = i;
     return true;
   }
@@ -315,7 +328,79 @@ bool Nibbler::getUnsignedInt (int& result)
 
   if (i > mCursor)
   {
-    result = atoi (mInput.substr (mCursor, i - mCursor).c_str ());
+    result = strtoimax (mInput.substr (mCursor, i - mCursor).c_str (), NULL, 10);
+    mCursor = i;
+    return true;
+  }
+
+  return false;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// number:
+//   int frac? exp?
+// 
+// int:
+//   -? digit+
+// 
+// frac:
+//   . digit+
+// 
+// exp:
+//   e digit+
+// 
+// e:
+//   e|E (+|-)?
+// 
+bool Nibbler::getNumber (double& result)
+{
+  std::string::size_type i = mCursor;
+
+  // [+-]?
+  if (i < mLength && mInput[i] == '-')
+    ++i;
+
+  // digit+
+  if (i < mLength && isdigit (mInput[i]))
+  {
+    ++i;
+
+    while (i < mLength && isdigit (mInput[i]))
+      ++i;
+
+    // ( . digit+ )?
+    if (i < mLength && mInput[i] == '.')
+    {
+      ++i;
+
+      while (i < mLength && isdigit (mInput[i]))
+        ++i;
+    }
+
+    // ( [eE] [+-]? digit+ )?
+    if (i < mLength && (mInput[i] == 'e' || mInput[i] == 'E'))
+    {
+      ++i;
+
+      if (i < mLength && (mInput[i] == '+' || mInput[i] == '-'))
+        ++i;
+
+      if (i < mLength && isdigit (mInput[i]))
+      {
+        ++i;
+
+        while (i < mLength && isdigit (mInput[i]))
+          ++i;
+
+        result = strtof (mInput.substr (mCursor, i - mCursor).c_str (), NULL);
+        mCursor = i;
+        return true;
+      }
+
+      return false;
+    }
+
+    result = strtof (mInput.substr (mCursor, i - mCursor).c_str (), NULL);
     mCursor = i;
     return true;
   }
@@ -467,6 +552,12 @@ char Nibbler::next ()
   return '\0';
 }
 
+////////////////////////////////////////////////////////////////////////////////
+std::string::size_type Nibbler::cursor ()
+{
+  return mCursor;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // Peeks ahead - does not move cursor.
 std::string Nibbler::next (const int quantity)
