@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -47,9 +47,15 @@ public:
   bool getUntilEOL (std::string&);
   bool getUntilEOS (std::string&);
 
-  bool getQuoted (char, std::string&, bool unescape = true, bool quote = false);
+/*
+  bool getAllOneOf (const std::string&, std::string&);
+*/
+
+  bool getQuoted (char, std::string&, bool quote = false);
   bool getInt (int&);
+  bool getHex (int&);
   bool getUnsignedInt (int&);
+  bool getNumber (double&);
   bool getLiteral (const std::string&);
   bool getRx (const std::string&, std::string&);
 
@@ -63,6 +69,8 @@ public:
   char next ();
   std::string next (const int quantity);
 
+  std::string::size_type cursor ();
+
   void save ();
   void restore ();
 
