@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -131,8 +131,9 @@ bool Record::has (const std::string& name) const
 std::vector <Att> Record::all ()
 {
   std::vector <Att> all;
-  foreach (a, (*this))
-    all.push_back (a->second);
+  std::map <std::string, Att>::iterator i;
+  for (i = this->begin (); i != this->end (); ++i)
+    all.push_back (i->second);
 
   return all;
 }
