@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -309,6 +309,8 @@ int TDB::loadPending (std::vector <Task>& tasks, Filter& filter)
           {
             // TODO Add hidden attribute indicating source?
             Task task (line);
+
+            Task::status status = task.getStatus ();
             task.id = mId++;
 
             mPending.push_back (task);
@@ -596,6 +598,10 @@ int TDB::gc ()
 {
   Timer t ("TDB::gc");
 
+  // Allowed as a temporary override.
+  if (!context.config.getBoolean ("gc"))
+    return 0;
+
   int count_pending_changes = 0;
   int count_completed_changes = 0;
   Date now;
@@ -993,7 +999,8 @@ void TDB::undo ()
   }
 
   // Output displayed, now confirm.
-  if (!confirm ("The undo command is not reversible.  Are you sure you want to revert to the previous state?"))
+  if (context.config.getBoolean ("confirmation") &&
+      !confirm ("The undo command is not reversible.  Are you sure you want to revert to the previous state?"))
   {
     std::cout << "No changes made.\n";
     context.hooks.trigger ("post-undo");
@@ -1090,7 +1097,7 @@ void TDB::undo ()
 void TDB::merge (const std::string& mergeFile)
 {
   ///////////////////////////////////////
-  // Copyright 2010, Johannes Schlatow.
+  // Copyright 2010 - 2011, Johannes Schlatow.
   ///////////////////////////////////////
 
   // list of modifications that we want to add to the local database
@@ -1450,7 +1457,7 @@ void TDB::merge (const std::string& mergeFile)
           // Find the same uuid in completed data
           for (it = completed.begin (); it != completed.end (); ++it)
           {
-            if (it->find (uuid) != std::string::npos)
+            if (it->find ("uuid:\"" + uuid) != std::string::npos)
             {
               // Update the completed record.
 /*
@@ -1493,7 +1500,7 @@ void TDB::merge (const std::string& mergeFile)
           // Find the same uuid in the pending data.
           for (it = pending.begin (); it != pending.end (); ++it)
           {
-            if (it->find (uuid) != std::string::npos)
+            if (it->find ("uuid:\"" + uuid) != std::string::npos)
             {
               // Update the pending record.
               std::cout << "Found remote change to        "
@@ -1548,7 +1555,7 @@ void TDB::merge (const std::string& mergeFile)
         std::vector <std::string>::iterator pit;
         for (pit = pending.begin (); pit != pending.end (); ++pit)
         {
-          if (pit->find (uuid) != std::string::npos)
+          if (pit->find ("uuid:\"" + uuid) != std::string::npos)
           {
             found = true;
             break;
