@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -763,8 +763,8 @@ bool sort_compare (int left, int right)
           return true;
         else if ((std::string)*cell_left != "" && (std::string)*cell_right == "")
           return false;
-        else if (Duration ((std::string)*cell_left) < Duration ((std::string)*cell_right))
-          return true;
+        else
+          return Duration ((std::string)*cell_left) < Duration ((std::string)*cell_right) ? true : false;
         break;
 
       case Table::descendingPeriod:
@@ -772,8 +772,8 @@ bool sort_compare (int left, int right)
           return false;
         else if ((std::string)*cell_left == "" && (std::string)*cell_right != "")
           return true;
-        else if (Duration ((std::string)*cell_left) < Duration ((std::string)*cell_right))
-          return false;
+        else
+          return Duration ((std::string)*cell_left) < Duration ((std::string)*cell_right) ? false : true;
         break;
       }
     }
