@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -27,14 +27,15 @@
 
 #include <sstream>
 #include <algorithm>
-#include "Context.h"
-#include "Nibbler.h"
-#include "Date.h"
-#include "Duration.h"
-#include "Task.h"
-#include "text.h"
-#include "util.h"
-#include "main.h"
+#include <Context.h>
+#include <Nibbler.h>
+#include <Date.h>
+#include <Duration.h>
+#include <Task.h>
+#include <JSON.h>
+#include <text.h>
+#include <util.h>
+#include <main.h>
 
 extern Context context;
 
@@ -81,6 +82,7 @@ bool Task::operator== (const Task& other)
 ////////////////////////////////////////////////////////////////////////////////
 Task::Task (const std::string& input)
 {
+  id = 0;
   parse (input);
 }
 
@@ -423,6 +425,108 @@ std::string Task::composeYAML () const
   return out.str ();
 }
 
+////////////////////////////////////////////////////////////////////////////////
+std::string Task::composeJSON () const
+{
+  std::stringstream out;
+  out << "{";
+
+  // Used for determining type.
+  Att att;
+
+  // First the non-annotations.
+  int attributes_written = 0;
+  int annotation_count = 0;
+  std::map <std::string, Att>::const_iterator i;
+  for (i = this->begin (); i != this->end (); ++i)
+  {
+    if (attributes_written)
+      out << ",";
+
+    // Annotations are simply counted.
+    if (i->second.name ().substr (0, 11) == "annotation_")
+    {
+      ++annotation_count;
+    }
+
+    // Date fields are written as ISO 8601.
+    else if (att.type (i->second.name ()) == "date")
+    {
+      Date d (i->second.value ());
+      out << "\""
+          << i->second.name ()
+          << "\":\""
+          << d.toISO ()
+          << "\"";
+
+      ++attributes_written;
+    }
+
+    // Tags are converted to an array.
+    else if (i->second.name () == "tags")
+    {
+      std::vector <std::string> tags;
+      split (tags, i->second.value (), ',');
+
+      out << "\"tags\":[";
+
+      std::vector <std::string>::iterator i;
+      for (i = tags.begin (); i != tags.end (); ++i)
+      {
+        if (i != tags.begin ())
+          out << ",";
+
+        out << "\"" << *i << "\"";
+      }
+
+      out << "]";
+    }
+
+    // Everything else is a quoted value.
+    else
+    {
+      out << "\""
+          << i->second.name ()
+          << "\":\""
+          << JSON::encode (i->second.value ())
+          << "\"";
+
+      ++attributes_written;
+    }
+  }
+
+  // Now the annotations, if any.
+  if (annotation_count)
+  {
+    out << ","
+        << "\"annotations\":[";
+
+    int annotations_written = 0;
+    for (i = this->begin (); i != this->end (); ++i)
+    {
+      if (i->second.name ().substr (0, 11) == "annotation_")
+      {
+        if (annotations_written)
+          out << ",";
+
+        Date d (i->second.name ().substr (11));
+        out << "{\"entry\":\""
+            << d.toISO ()
+            << "\",\"description\":\""
+            << JSON::encode (i->second.value ())
+            << "\"}";
+
+        ++annotations_written;
+      }
+    }
+
+    out << "]";
+  }
+
+  out << "}";
+  return out.str ();
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 void Task::getAnnotations (std::vector <Att>& annotations) const
 {
