@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -42,11 +42,10 @@
 #include "text.h"
 #include "util.h"
 #include "main.h"
-#include "../auto.h"
 #include "Transport.h"
-
-#ifdef HAVE_LIBNCURSES
-#include <ncurses.h>
+#include "../auto.h"
+#ifdef HAVE_COMMIT
+#include "../commit.h"
 #endif
 
 extern Context context;
@@ -91,6 +90,15 @@ int handleAdd (std::string& outs)
         context.task.set ("priority", defaultPriority);
     }
 
+    // Override with default.due, if not specified.
+    if (context.task.get ("due") == "")
+    {
+      std::string defaultDue = context.config.get ("default.due");
+      if (defaultDue != "" &&
+          Att::validNameValue ("due", "", defaultDue))
+        context.task.set ("due", defaultDue);
+    }
+
     // Include tags.
     foreach (tag, context.tagAdditions)
       context.task.addTag (*tag);
@@ -185,6 +193,15 @@ int handleLog (std::string& outs)
         context.task.set ("priority", defaultPriority);
     }
 
+    // Override with default.due, if not specified.
+    if (context.task.get ("due") == "")
+    {
+      std::string defaultDue = context.config.get ("default.due");
+      if (defaultDue != "" &&
+          Att::validNameValue ("due", "", defaultDue))
+        context.task.set ("due", defaultDue);
+    }
+
     // Include tags.
     foreach (tag, context.tagAdditions)
       context.task.addTag (*tag);
@@ -195,12 +212,12 @@ int handleLog (std::string& outs)
     context.tdb.lock (context.config.getBoolean ("locking"));
     context.tdb.add (context.task);
     context.tdb.commit ();
-    context.tdb.unlock ();
 
     if (context.config.getBoolean ("echo.command"))
       out << "Logged task.\n";
 
     context.footnote (onProjectChange (context.task));
+    context.tdb.unlock ();
 
     outs = out.str ();
     context.hooks.trigger ("post-log-command");
@@ -236,6 +253,7 @@ int handleProjects (std::string& outs)
     std::map <std::string, int> medium;
     std::map <std::string, int> low;
     std::map <std::string, int> none;
+    bool no_project = false;
     std::string project;
     std::string priority;
     foreach (t, tasks)
@@ -244,6 +262,8 @@ int handleProjects (std::string& outs)
       priority = t->get ("priority");
 
       unique[project] += 1;
+      if (project == "")
+        no_project = true;
 
            if (priority == "H") high[project]   += 1;
       else if (priority == "M") medium[project] += 1;
@@ -284,7 +304,7 @@ int handleProjects (std::string& outs)
       foreach (i, unique)
       {
         int row = table.addRow ();
-        table.addCell (row, 0, i->first);
+        table.addCell (row, 0, (i->first == "" ? "(none)" : i->first));
         table.addCell (row, 1, i->second);
         table.addCell (row, 2, none[i->first]);
         table.addCell (row, 3, low[i->first]);
@@ -292,11 +312,15 @@ int handleProjects (std::string& outs)
         table.addCell (row, 5, high[i->first]);
       }
 
+      int number_projects = unique.size ();
+      if (no_project)
+        --number_projects;
+
       out << optionalBlankLine ()
           << table.render ()
           << optionalBlankLine ()
-          << unique.size ()
-          << (unique.size () == 1 ? " project" : " projects")
+          << number_projects
+          << (number_projects == 1 ? " project" : " projects")
           << " (" << quantity << (quantity == 1 ? " task" : " tasks") << ")\n";
     }
     else
@@ -503,7 +527,11 @@ int handleCompletionConfig (std::string& outs)
 // script.
 int handleCompletionVersion (std::string& outs)
 {
+#ifdef HAVE_COMMIT
+  outs = COMMIT;
+#else
   outs = VERSION;
+#endif
   outs += "\n";
   return 0;
 }
@@ -522,6 +550,11 @@ int handleUrgency (std::string& outs)
 
   // Filter sequence.
   context.filter.applySequence (tasks, context.sequence);
+  if (tasks.size () == 0)
+  {
+    std::cout << "No tasks specified.\n";
+    return 1;
+  }
 
   // Find the task(s).
   std::stringstream out;
@@ -538,6 +571,50 @@ int handleUrgency (std::string& outs)
   return 0;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+int handleQuery (std::string& outs)
+{
+  int rc = 0;
+
+  if (context.hooks.trigger ("pre-query-command"))
+  {
+    // Get all the tasks.
+    std::vector <Task> tasks;
+    context.tdb.lock (context.config.getBoolean ("locking"));
+    handleRecurrence ();
+    context.tdb.load (tasks, context.filter);
+    context.tdb.commit ();
+    context.tdb.unlock ();
+
+    // Filter sequence.
+    if (context.sequence.size ())
+      context.filter.applySequence (tasks, context.sequence);
+
+    if (tasks.size () == 0)
+    {
+      std::cout << "No matches.\n";
+      return 1;
+    }
+
+    // Note: "limit:" feature not supported.
+
+    // Compose output.
+    std::vector <Task>::iterator t;
+    for (t = tasks.begin (); t != tasks.end (); ++t)
+    {
+      if (t != tasks.begin ())
+        outs += ",\n";
+
+      outs += t->composeJSON ();
+    }
+
+    outs += "\n";
+    context.hooks.trigger ("post-query-command");
+  }
+
+  return rc;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 int handleCompletionIDs (std::string& outs)
 {
@@ -564,6 +641,199 @@ int handleCompletionIDs (std::string& outs)
   return 0;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+int handleZshCompletionIDs (std::string& outs)
+{
+  std::vector <Task> tasks;
+  context.tdb.lock (context.config.getBoolean ("locking"));
+  Filter filter;
+  context.tdb.loadPending (tasks, filter);
+  context.tdb.commit ();
+  context.tdb.unlock ();
+
+  std::stringstream out;
+  foreach (task, tasks) {
+    if (task->getStatus () != Task::deleted &&
+        task->getStatus () != Task::completed) {
+      out << task->id << ":" << task->get("description") << "\n";
+    }
+  }
+
+  outs = out.str ();
+  return 0;
+}
+
+
+////////////////////////////////////////////////////////////////////////////////
+int handleZshCompletionCommands (std::string& outs)
+{
+  // Get a list of all commands.
+  std::vector <std::string> commands;
+  context.cmd.allCommands (commands);
+
+  // Sort alphabetically.
+  std::sort (commands.begin (), commands.end ());
+
+  std::stringstream out;
+  foreach (command, commands) {
+    out << *command << ":";
+
+    if (*command == "add") {
+      out << "Adds a new task.";
+    }
+    else if (*command == "log") {
+      out << "Adds a new task that is already completed.";
+    }
+    else if (*command == "append") {
+      out << "Appends more description to an existing task.";
+    }
+    else if (*command == "prepend") {
+      out << "Prepends more description to an existing task.";
+    }
+    else if (*command == "annotate") {
+      out << "Adds an annotation to an existing task.";
+    }
+    else if (*command == "denotate") {
+      out << "Deletes an annotation of an existing task.";
+    }
+    else if (*command == "edit") {
+      out << "Launches an editor to let you modify a task directly.";
+    }
+    else if (*command == "undo") {
+      out << "Reverts the most recent action.";
+    }
+    else if (*command == "shell") {
+      out << "Launches an interactive shell.";
+    }
+    else if (*command == "duplicate") {
+      out <<  "Duplicates the specified task, and allows modifications.";
+    }
+    else if (*command == "delete") {
+      out << "Deletes the specified task.";
+    }
+    else if (*command == "info") {
+      out << "Shows all data, metadata for specified task.";
+    }
+    else if (*command == "start") {
+      out << "Marks specified task as started.";
+    }
+    else if (*command == "stop") {
+      out << "Removes the 'start' time from a task.";
+    }
+    else if (*command == "done") {
+      out << "Marks the specified task as completed.";
+    }
+    else if (*command == "projects") {
+      out << "Shows a list of all project names used.";
+    }
+    else if (*command == "tags") {
+      out << "Shows a list of all tags used.";
+    }
+    else if (*command == "summary") {
+      out << "Shows a report of task status by project.";
+    }
+    else if (*command == "timesheet") {
+      out << "Shows a weekly report of tasks completed and started.";
+    }
+    else if (*command == "history") {
+      out << "Alias to history.monthly.";
+    }
+    else if (*command == "history.monthly") {
+      out << "Shows a report of task history, by month.";
+    }
+    else if (*command == "history.annual") {
+      out << "Shows a report of task history, by year.";
+    }
+    else if (*command == "ghistory") {
+      out <<  "Alias to ghistory.monthly.";
+    }
+    else if (*command == "ghistory.monthly") {
+      out <<  "Shows a graphical report of task history, by month.";
+    }
+    else if (*command == "ghistory.annual") {
+      out << "Shows a graphical report of task history, by year.";
+    }
+    else if (*command == "burndown") {
+      out << "Alias to burndown.weekly.";
+    }
+    else if (*command == "burndown.daily") {
+      out << "Shows a graphical burndown chart, by day.";
+    }
+    else if (*command == "burndown.weekly") {
+      out << "Shows a graphical burndown chart, by week.";
+    }
+    else if (*command == "burndown.monthly") {
+      out << "Shows a graphical burndown chart, by month.";
+    }
+    else if (*command == "calendar") {
+      out << "Shows a calendar, with due tasks marked.";
+    }
+    else if (*command == "stats") {
+      out << "Shows task database statistics.";
+    }
+    else if (*command == "import") {
+      out << "Imports tasks from a variety of formats.";
+    }
+    else if (*command == "export") {
+      out << "Alias to export.csv.";
+    }
+    else if (*command == "export.csv") {
+      out << "Lists all tasks in CSV format.";
+    }
+    else if (*command == "export.ical") {
+      out << "Lists all tasks in iCalendar format.";
+    }
+    else if (*command == "export.vcalendar") {
+      out << "Lists all tasks in vCalendar format.";
+    }
+    else if (*command == "export.yaml") {
+      out << "Lists all tasks in YAML format.";
+    }
+    else if (*command == "merge") {
+      out << "Merges the specified database with the local database.";
+    }
+    else if (*command == "push") {
+      out << "Pushed the local database to the specified URI.";
+    }
+    else if (*command == "pull") {
+      out << "Overwrites the local database with that found at the URI.";
+    }
+    else if (*command == "colors") {
+      out << "Displays all possible colors or a named sample [legend | sample].";
+    }
+    else if (*command == "count") {
+      out << "Shows only the number of matching tasks.";
+    }
+    else if (*command == "version") {
+      out << "Shows the task version number.";
+    }
+    else if (*command == "show") {
+      out << "Shows the entire task configuration variables.";
+    }
+    else if (*command == "config") {
+      out << "Add, modify and remove settings in the task configuration.";
+    }
+    else if (*command == "diagnostics") {
+      out << "Information needed when reporting a problem.";
+    }
+    else if (*command == "help") {
+      out << "Shows the long usage text.";
+    }
+    else if (*command == "rm") {
+      out << "Alias to delete.";
+    }
+    else {
+      // try to interpret this as custom report
+      out << context.config.get (std::string ("report." + *command + ".description"));
+    }
+
+    out << "\n";
+  }
+
+  outs = out.str ();
+  return 0;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 void handleUndo ()
 {
@@ -594,13 +864,6 @@ void handleMerge (std::string& outs)
     Uri uri (file, "merge");
     uri.parse();
 
-    if (sAutopush == "ask")
-    {
-      // expand uri
-      Uri push (file, "push");
-      pushfile = push.data;
-    }
-
     if (uri.data.length ())
     {
       Directory location (context.config.get ("data.location"));
@@ -631,10 +894,11 @@ void handleMerge (std::string& outs)
       if (tmpfile != "")
         remove (tmpfile.c_str ());
 
-      if ( ((sAutopush == "ask") && (confirm ("Would you like to push the merged changes to \'" + pushfile + "\'?")) )
+      if ( ((sAutopush == "ask") && (confirm ("Would you like to push the merged changes to \'" + uri.data + "\'?")) )
          || (bAutopush) )
       {
         std::string out;
+		  context.task.set ("description", uri.data);
         handlePush (out);
       }
     }
@@ -836,10 +1100,6 @@ int handleVersion (std::string& outs)
         << "unknown"
 #endif
 
-#ifdef HAVE_LIBNCURSES
-        << "-ncurses"
-#endif
-
 #ifdef HAVE_LIBREADLINE
         << "-readline"
 #endif
@@ -849,7 +1109,7 @@ int handleVersion (std::string& outs)
 #endif
 
         << "\n"
-        << "Copyright (C) 2006 - 2010 P. Beckingham, F. Hernandez.\n"
+        << "Copyright (C) 2006 - 2011 P. Beckingham, F. Hernandez.\n"
 #ifdef HAVE_LIBLUA
         << "Portions of this software Copyright (C) 1994 – 2008 Lua.org, PUC-Rio.\n"
 #endif
@@ -891,28 +1151,30 @@ int handleShow (std::string& outs)
     // Note that there is a leading and trailing space, to make it easier to
     // search for whole words.
     std::string recognized =
-      " annotations blanklines bulk calendar.details calendar.details.report "
-      "calendar.holidays calendar.legend color color.active color.due "
-      "color.due.today color.blocked color.overdue color.pri.H color.pri.L "
-      "color.pri.M color.pri.none color.recurring color.tagged color.footnote "
-      "color.header color.debug color.alternate color.calendar.today "
-      "color.calendar.due color.calendar.due.today color.calendar.overdue "
+      " annotations blanklines bulk burndown.bias calendar.details calendar.details.report "
+      "calendar.holidays calendar.legend color calendar.offset calendar.offset.value "
+      "color.active color.due color.due.today color.blocked color.burndown.done "
+      "color.burndown.pending color.burndown.started color.overdue color.pri.H "
+      "color.pri.L color.pri.M color.pri.none color.recurring color.tagged "
+      "color.footnote color.header color.debug color.alternate color.calendar.today "
+      "color.calendar.due color.calendar.due.today color.calendar.overdue regex "
       "color.calendar.weekend color.calendar.holiday color.calendar.weeknumber "
       "color.summary.background color.summary.bar color.history.add "
       "color.history.done color.history.delete color.undo.before "
       "color.sync.added color.sync.changed color.sync.rejected "
       "color.undo.after confirmation curses data.location dateformat "
       "dateformat.holiday dateformat.report dateformat.annotation debug "
-      "default.command default.priority default.project defaultwidth due "
+      "default.command default.due default.priority default.project defaultwidth due "
       "dependency.confirmation dependency.reminder locale displayweeknumber "
-      "export.ical.class echo.command fontunderline locking monthsperline nag "
-      "next journal.time journal.time.start.annotation "
+      "export.ical.class echo.command fontunderline gc locking monthsperline "
+      "nag next journal.time journal.time.start.annotation journal.info "
       "journal.time.stop.annotation project shadow.command shadow.file "
-      "shadow.notify weekstart editor import.synonym.id import.synonym.uuid "
+      "shadow.notify weekstart editor edit.verbose import.synonym.id import.synonym.uuid "
       "complete.all.projects complete.all.tags search.case.sensitive hooks "
       "active.indicator tag.indicator recurrence.indicator recurrence.limit "
       "list.all.projects list.all.tags undo.style verbose rule.precedence.color "
       "merge.autopush merge.default.uri pull.default.uri push.default.uri "
+      "xterm.title "
 #ifdef FEATURE_SHELL
       "shell.prompt "
 #endif
@@ -957,6 +1219,15 @@ int handleShow (std::string& outs)
       }
     }
 
+    // Find all the values that match the defaults, for highlighting.
+    std::vector <std::string> default_values;
+    Config default_config;
+    default_config.setDefaults ();
+
+    foreach (i, all)
+      if (context.config.get (*i) != default_config.get (*i))
+        default_values.push_back (*i);
+
     // Create a table for output.
     Table table;
     table.setTableWidth (width);
@@ -979,6 +1250,7 @@ int handleShow (std::string& outs)
     table.sortOn (0, Table::ascendingCharacter);
 
     Color error ("bold white on red");
+    Color warning ("black on yellow");
 
     std::string section;
 
@@ -1000,8 +1272,13 @@ int handleShow (std::string& outs)
 
         // Look for unrecognized.
         if (context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor"))
+        {
           if (std::find (unrecognized.begin (), unrecognized.end (), *i) != unrecognized.end ())
             table.setRowColor (row, error);
+
+          else if (std::find (default_values.begin (), default_values.end (), *i) != default_values.end ())
+            table.setRowColor (row, warning);
+        }
       }
     }
 
@@ -1020,7 +1297,15 @@ int handleShow (std::string& outs)
       if (context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor"))
         out << "\n  These are highlighted in " << error.colorize ("color") << " above.";
 
-      out << "\n";
+      out << "\n\n";
+    }
+
+    if (default_values.size ())
+    {
+      out << "Some of your .taskrc variables differ from the default values.";
+
+      if (context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor"))
+        out << "  These are highlighted in " << warning.colorize ("color") << " above.";
     }
 
     out << context.config.checkForDeprecatedColor ();
@@ -1276,6 +1561,11 @@ int handleDelete (std::string& outs)
     // Filter sequence.
     std::vector <Task> all = tasks;
     context.filter.applySequence (tasks, context.sequence);
+    if (tasks.size () == 0)
+    {
+      std::cout << "No tasks specified.\n";
+      return 1;
+    }
 
     // Determine the end date.
     char endTime[16];
@@ -1283,54 +1573,78 @@ int handleDelete (std::string& outs)
 
     foreach (task, tasks)
     {
-      if (context.hooks.trigger ("pre-delete", *task))
+      if (task->getStatus () == Task::pending ||
+          task->getStatus () == Task::waiting)
       {
-        std::stringstream question;
-        question << "Permanently delete task "
-                 << task->id
-                 << " '"
-                 << task->get ("description")
-                 << "'?";
-
-        if (!context.config.getBoolean ("confirmation") || confirm (question.str ()))
+        if (context.hooks.trigger ("pre-delete", *task))
         {
-          // Check for the more complex case of a recurring task.  If this is a
-          // recurring task, get confirmation to delete them all.
-          std::string parent = task->get ("parent");
-          if (parent != "")
+          std::stringstream question;
+          question << "Permanently delete task "
+                   << task->id
+                   << " '"
+                   << task->get ("description")
+                   << "'?";
+
+          if (!context.config.getBoolean ("confirmation") || confirm (question.str ()))
           {
-            if (confirm ("This is a recurring task.  Do you want to delete all pending recurrences of this same task?"))
+            // Check for the more complex case of a recurring task.  If this is a
+            // recurring task, get confirmation to delete them all.
+            std::string parent = task->get ("parent");
+            if (parent != "")
             {
-              // Scan all pending tasks for siblings of this task, and the parent
-              // itself, and delete them.
-              foreach (sibling, all)
+              if (confirm ("This is a recurring task.  Do you want to delete all pending recurrences of this same task?"))
               {
-                if (sibling->get ("parent") == parent ||
-                    sibling->get ("uuid")   == parent)
+                // Scan all pending tasks for siblings of this task, and the parent
+                // itself, and delete them.
+                foreach (sibling, all)
                 {
-                  sibling->setStatus (Task::deleted);
+                  if (sibling->get ("parent") == parent ||
+                      sibling->get ("uuid")   == parent)
+                  {
+                    sibling->setStatus (Task::deleted);
 
-                  // Don't want a 'delete' to clobber the end date that may have
-                  // been written by a 'done' command.
-                  if (! sibling->has ("end"))
-                    sibling->set ("end", endTime);
+                    // Don't want a 'delete' to clobber the end date that may have
+                    // been written by a 'done' command.
+                    if (! sibling->has ("end"))
+                      sibling->set ("end", endTime);
 
-                  context.tdb.update (*sibling);
+                    context.tdb.update (*sibling);
 
-                  if (context.config.getBoolean ("echo.command"))
-                    out << "Deleting recurring task "
-                        << sibling->id
-                        << " '"
-                        << sibling->get ("description")
-                        << "'.\n";
+                    if (context.config.getBoolean ("echo.command"))
+                      out << "Deleting recurring task "
+                          << sibling->id
+                          << " '"
+                          << sibling->get ("description")
+                          << "'.\n";
+                  }
                 }
               }
+              else
+              {
+                // Update mask in parent.
+                task->setStatus (Task::deleted);
+                updateRecurrenceMask (all, *task);
+
+                // Don't want a 'delete' to clobber the end date that may have
+                // been written by a 'done' command.
+                if (! task->has ("end"))
+                  task->set ("end", endTime);
+
+                context.tdb.update (*task);
+
+                out << "Deleting recurring task "
+                    << task->id
+                    << " '"
+                    << task->get ("description")
+                    << "'.\n";
+
+                dependencyChainOnComplete (*task);
+                context.footnote (onProjectChange (*task));
+              }
             }
             else
             {
-              // Update mask in parent.
               task->setStatus (Task::deleted);
-              updateRecurrenceMask (all, *task);
 
               // Don't want a 'delete' to clobber the end date that may have
               // been written by a 'done' command.
@@ -1339,11 +1653,12 @@ int handleDelete (std::string& outs)
 
               context.tdb.update (*task);
 
-              out << "Deleting recurring task "
-                  << task->id
-                  << " '"
-                  << task->get ("description")
-                  << "'.\n";
+              if (context.config.getBoolean ("echo.command"))
+                out << "Deleting task "
+                    << task->id
+                    << " '"
+                    << task->get ("description")
+                    << "'.\n";
 
               dependencyChainOnComplete (*task);
               context.footnote (onProjectChange (*task));
@@ -1351,32 +1666,21 @@ int handleDelete (std::string& outs)
           }
           else
           {
-            task->setStatus (Task::deleted);
-
-            // Don't want a 'delete' to clobber the end date that may have
-            // been written by a 'done' command.
-            if (! task->has ("end"))
-              task->set ("end", endTime);
-
-            context.tdb.update (*task);
-
-            if (context.config.getBoolean ("echo.command"))
-              out << "Deleting task "
-                  << task->id
-                  << " '"
-                  << task->get ("description")
-                  << "'.\n";
-
-            dependencyChainOnComplete (*task);
-            context.footnote (onProjectChange (*task));
+            out << "Task not deleted.\n";
+            rc  = 1;
           }
-        }
-        else {
-          out << "Task not deleted.\n";
-          rc  = 1;
-        }
 
-        context.hooks.trigger ("post-delete", *task);
+          context.hooks.trigger ("post-delete", *task);
+        }
+      }
+      else
+      {
+        out << "Task "
+            << task->id
+            << " '"
+            << task->get ("description")
+            << "' is neither pending nor waiting.\n";
+        rc = 1;
       }
     }
 
@@ -1408,6 +1712,11 @@ int handleStart (std::string& outs)
 
     // Filter sequence.
     context.filter.applySequence (tasks, context.sequence);
+    if (tasks.size () == 0)
+    {
+      std::cout << "No tasks specified.\n";
+      return 1;
+    }
 
     bool nagged = false;
     foreach (task, tasks)
@@ -1474,6 +1783,11 @@ int handleStop (std::string& outs)
 
     // Filter sequence.
     context.filter.applySequence (tasks, context.sequence);
+    if (tasks.size () == 0)
+    {
+      std::cout << "No tasks specified.\n";
+      return 1;
+    }
 
     foreach (task, tasks)
     {
@@ -1532,6 +1846,11 @@ int handleDone (std::string& outs)
     // Filter sequence.
     std::vector <Task> all = tasks;
     context.filter.applySequence (tasks, context.sequence);
+    if (tasks.size () == 0)
+    {
+      std::cout << "No tasks specified.\n";
+      return 1;
+    }
 
     Permission permission;
     if (context.sequence.size () > (size_t) context.config.getInteger ("bulk"))
@@ -1561,6 +1880,11 @@ int handleDone (std::string& outs)
         // Change status.
         task->setStatus (Task::completed);
 
+        // Stop the task, if started.
+        if (task->has ("start") &&
+            context.config.getBoolean ("journal.time"))
+          task->addAnnotation (context.config.get ("journal.time.stop.annotation"));
+
         // Only allow valid tasks.
         task->validate ();
 
@@ -1595,12 +1919,14 @@ int handleDone (std::string& outs)
           nagged = nag (*task);
       }
       else
+      {
         out << "Task "
             << task->id
             << " '"
             << task->get ("description")
             << "' is neither pending nor waiting.\n";
         rc = 1;
+      }
     }
 
     if (count)
@@ -1638,6 +1964,11 @@ int handleModify (std::string& outs)
   // Filter sequence.
   std::vector <Task> all = tasks;
   context.filter.applySequence (tasks, context.sequence);
+  if (tasks.size () == 0)
+  {
+    std::cout << "No tasks specified.\n";
+    return 1;
+  }
 
   Permission permission;
   if (context.sequence.size () > (size_t) context.config.getInteger ("bulk"))
@@ -1667,29 +1998,6 @@ int handleModify (std::string& outs)
         context.task.get ("recur") == "")
       throw std::string ("You cannot remove the recurrence from a recurring task.");
 
-    // This looks unnecessarily complex, but isn't.  "due" and "wait" are
-    // independent and may exist without the other, but if both exist then wait
-    // must be before due.
-    if ((task->has ("wait")       &&
-         context.task.has ("due") &&
-         Date (context.task.get ("due")) < Date (task->get ("wait")))
-        ||
-        (context.task.has ("wait") &&
-         !context.task.has ("due") &&
-         task->has ("due")         &&
-         Date (task->get ("due")) < Date (context.task.get ("wait")))
-        ||
-        (context.task.has ("wait") &&
-         context.task.has ("due")  &&
-         Date (context.task.get ("due")) < Date (context.task.get ("wait")))
-        ||
-        (task->has ("wait") &&
-         task->has ("due")  &&
-         Date (task->get ("due")) < Date (task->get ("wait"))))
-    {
-      context.footnote ("Warning: the wait date falls after the due date.");
-    }
-
     // Make all changes.
     bool warned = false;
     foreach (other, all)
@@ -1796,6 +2104,11 @@ int handleAppend (std::string& outs)
     // Filter sequence.
     std::vector <Task> all = tasks;
     context.filter.applySequence (tasks, context.sequence);
+    if (tasks.size () == 0)
+    {
+      std::cout << "No tasks specified.\n";
+      return 1;
+    }
 
     Permission permission;
     if (context.sequence.size () > (size_t) context.config.getInteger ("bulk"))
@@ -1878,6 +2191,11 @@ int handlePrepend (std::string& outs)
     // Filter sequence.
     std::vector <Task> all = tasks;
     context.filter.applySequence (tasks, context.sequence);
+    if (tasks.size () == 0)
+    {
+      std::cout << "No tasks specified.\n";
+      return 1;
+    }
 
     Permission permission;
     if (context.sequence.size () > (size_t) context.config.getInteger ("bulk"))
@@ -1959,6 +2277,11 @@ int handleDuplicate (std::string& outs)
 
     // Filter sequence.
     context.filter.applySequence (tasks, context.sequence);
+    if (tasks.size () == 0)
+    {
+      std::cout << "No tasks specified.\n";
+      return 1;
+    }
 
     foreach (task, tasks)
     {
@@ -2010,7 +2333,12 @@ int handleDuplicate (std::string& outs)
       ++count;
     }
 
-    if (context.config.getBoolean ("echo.command"))
+    if (tasks.size () == 0)
+    {
+      out << "No matches.\n";
+      rc = 1;
+    }
+    else if (context.config.getBoolean ("echo.command"))
     {
 #ifdef FEATURE_NEW_ID
       // All this, just for an id number.
@@ -2031,6 +2359,37 @@ int handleDuplicate (std::string& outs)
   return rc;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+int handleCount (std::string& outs)
+{
+  int rc = 0;
+
+  if (context.hooks.trigger ("pre-count-command"))
+  {
+    // Scan the pending tasks, applying any filter.
+    std::vector <Task> tasks;
+    context.tdb.lock (context.config.getBoolean ("locking"));
+    handleRecurrence ();
+    context.tdb.load (tasks, context.filter);
+    context.tdb.commit ();
+    context.tdb.unlock ();
+
+    // Find number of matching tasks.  Skip recurring parent tasks.
+    int count = 0;
+    std::vector <Task>::iterator it;
+    for (it = tasks.begin (); it != tasks.end (); ++it)
+      if (it->getStatus () != Task::recurring)
+        ++count;
+
+    std::stringstream out;
+    out << count << "\n";
+    outs = out.str ();
+    context.hooks.trigger ("post-count-command");
+  }
+
+  return rc;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 #ifdef FEATURE_SHELL
 void handleShell ()
@@ -2045,13 +2404,17 @@ void handleShell ()
               << " shell\n\n"
               << "Enter any task command (such as 'list'), or hit 'Enter'.\n"
               << "There is no need to include the 'task' command itself.\n"
-              << "Enter 'quit' to end the session.\n\n";
+              << "Enter 'quit' (or 'bye', 'exit') to end the session.\n\n";
 
     // Make a copy because context.clear will delete them.
     std::string permanentOverrides = " " + context.file_override
                                    + " " + context.var_overrides;
 
-    std::string quit = "quit"; // TODO i18n
+    std::vector <std::string> quit_commands;
+    quit_commands.push_back ("quit");
+    quit_commands.push_back ("exit");
+    quit_commands.push_back ("bye");
+
     std::string command;
     bool keepGoing = true;
 
@@ -2071,9 +2434,7 @@ void handleShell ()
 
       // When looking for the 'quit' command, use 'command', not
       // 'decoratedCommand'.
-      if (command.length ()   >  0              &&
-          command.length ()   <= quit.length () &&
-          lowerCase (command) == quit.substr (0, command.length ()))
+      if (std::find (quit_commands.begin (), quit_commands.end (), lowerCase (command)) != quit_commands.end ())
       {
         keepGoing = false;
       }
@@ -2346,6 +2707,11 @@ int handleAnnotate (std::string& outs)
 
     // Filter sequence.
     context.filter.applySequence (tasks, context.sequence);
+    if (tasks.size () == 0)
+    {
+      std::cout << "No tasks specified.\n";
+      return 1;
+    }
 
     Permission permission;
     if (context.sequence.size () > (size_t) context.config.getInteger ("bulk"))
@@ -2408,6 +2774,11 @@ int handleDenotate (std::string& outs)
     context.tdb.loadPending (tasks, filter);
 
     context.filter.applySequence (tasks, context.sequence);
+    if (tasks.size () == 0)
+    {
+      std::cout << "No tasks specified.\n";
+      return 1;
+    }
 
     Permission permission;
     if (context.sequence.size () > (size_t) context.config.getInteger ("bulk"))
@@ -2633,3 +3004,4 @@ int deltaSubstitutions (Task& task)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
+// vim: ts=2 sw=2 et
