@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -43,10 +43,6 @@
 #include "util.h"
 #include "main.h"
 
-#ifdef HAVE_LIBNCURSES
-#include <ncurses.h>
-#endif
-
 extern Context context;
 static std::vector <std::string> customReports;
 
@@ -122,9 +118,16 @@ int handleCustomReport (const std::string& report, std::string& outs)
     if (context.sequence.size ())
       context.filter.applySequence (tasks, context.sequence);
 
+    // Determine the output date format, which uses a hierarchy of definitions.
+    std::string dateformat = context.config.get ("report." + report + ".dateformat");
+    if (dateformat == "")
+      dateformat = context.config.get ("dateformat.report");
+    if (dateformat == "")
+      dateformat = context.config.get ("dateformat");
+
     Table table;
     table.setTableWidth (context.getWidth ());
-    table.setDateFormat (context.config.get ("dateformat"));
+    table.setDateFormat (dateformat);
     table.setReportName (report);
 
     foreach (task, tasks)
@@ -144,20 +147,14 @@ int handleCustomReport (const std::string& report, std::string& outs)
         table.setColumnWidth (columnCount, Table::minimum);
         table.setColumnJustification (columnCount, Table::right);
 
-        char s[16];
         std::string value;
         int row = 0;
         foreach (task, tasks)
         {
           if (task->id != 0)
-          {
-            sprintf (s, "%d", (int) task->id);
-            value = s;
-          }
+            value = format (task->id);
           else
-          {
             value = "-";
-          }
 
           context.hooks.trigger ("format-id", "id", value);
           table.addCell (row++, columnCount, value);
@@ -245,7 +242,7 @@ int handleCustomReport (const std::string& report, std::string& outs)
           if (entered.length ())
           {
             Date dt (::atoi (entered.c_str ()));
-            entered = dt.toString (context.config.get ("dateformat"));
+            entered = dt.toString (dateformat);
             context.hooks.trigger ("format-entry", "entry", entered);
             table.addCell (row, columnCount, entered);
           }
@@ -265,7 +262,7 @@ int handleCustomReport (const std::string& report, std::string& outs)
           if (started.length ())
           {
             Date dt (::atoi (started.c_str ()));
-            started = dt.toString (context.config.get ("dateformat"));
+            started = dt.toString (dateformat);
             context.hooks.trigger ("format-start", "start", started);
             table.addCell (row, columnCount, started);
           }
@@ -285,7 +282,7 @@ int handleCustomReport (const std::string& report, std::string& outs)
           if (ended.length ())
           {
             Date dt (::atoi (ended.c_str ()));
-            ended = dt.toString (context.config.get ("dateformat"));
+            ended = dt.toString (dateformat);
             context.hooks.trigger ("format-end", "end", ended);
             table.addCell (row, columnCount, ended);
           }
@@ -298,17 +295,11 @@ int handleCustomReport (const std::string& report, std::string& outs)
         table.setColumnWidth (columnCount, Table::minimum);
         table.setColumnJustification (columnCount, Table::left);
 
-        std::string format = context.config.get ("report." + report + ".dateformat");
-        if (format == "")
-          format = context.config.get ("dateformat.report");
-        if (format == "")
-          format = context.config.get ("dateformat");
-
         int row = 0;
         std::string due;
         foreach (task, tasks)
         {
-          std::string value = getDueDate (*task, format);
+          std::string value = getDueDate (*task, dateformat);
           context.hooks.trigger ("format-due", "due", value);
           table.addCell (row++, columnCount, value);
         }
@@ -398,7 +389,6 @@ int handleCustomReport (const std::string& report, std::string& outs)
           {
             Date dt (::atoi (created.c_str ()));
             age = Duration (now - dt).formatCompact ();
-
             context.hooks.trigger ("format-age_compact", "age_compact", age);
             table.addCell (row, columnCount, age);
           }
@@ -519,7 +509,7 @@ int handleCustomReport (const std::string& report, std::string& outs)
           if (wait != "")
           {
             Date dt (::atoi (wait.c_str ()));
-            wait = dt.toString (context.config.get ("dateformat"));
+            wait = dt.toString (dateformat);
             context.hooks.trigger ("format-wait", "wait", wait);
             table.addCell (row++, columnCount, wait);
           }
@@ -530,7 +520,7 @@ int handleCustomReport (const std::string& report, std::string& outs)
       {
         table.addColumn (columnLabels[*col] != "" ? columnLabels[*col] : "Deps");
         table.setColumnWidth (columnCount, Table::minimum);
-        table.setColumnJustification (columnCount, Table::left);
+        table.setColumnJustification (columnCount, Table::right);
 
         int row = 0;
         std::vector <Task> blocked;
@@ -551,6 +541,21 @@ int handleCustomReport (const std::string& report, std::string& outs)
         }
       }
 
+      else if (*col == "urgency")
+      {
+        table.addColumn (columnLabels[*col] != "" ? columnLabels[*col] : "Urgency");
+        table.setColumnWidth (columnCount, Table::minimum);
+        table.setColumnJustification (columnCount, Table::right);
+
+        int row = 0;
+        foreach (task, tasks)
+        {
+          std::string value = format (task->urgency (), 4, 3);
+          context.hooks.trigger ("format-urgency", "urgency", value);
+          table.addCell (row++, columnCount, value);
+        }
+      }
+
       // Common to all columns.
       // Add underline.
       if ((context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor")) &&
@@ -575,7 +580,7 @@ int handleCustomReport (const std::string& report, std::string& outs)
       char direction = (*sortColumn)[sortColumn->length () - 1];
 
       // TODO This code should really be using Att::type.
-      if (column == "id")
+      if (column == "id" || column == "urgency")
         table.sortOn (columnIndex[column],
                       (direction == '+' ?
                         Table::ascendingNumeric :
@@ -639,6 +644,14 @@ int handleCustomReport (const std::string& report, std::string& outs)
         table.setTableAlternateColor (alternate);
     }
 
+    // How many lines taken up by table header?
+    int table_header;
+    if ((context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor")) &&
+        context.config.getBoolean ("fontunderline"))
+      table_header = 1;  // Underlining doesn't use extra line.
+    else
+      table_header = 2;  // Dashes use an extra line.
+
     // Report output can be limited by rows or lines.
     int maxrows = 0;
     int maxlines = 0;
@@ -647,7 +660,7 @@ int handleCustomReport (const std::string& report, std::string& outs)
     // Adjust for fluff in the output.
     if (maxlines)
       maxlines -= (context.config.getBoolean ("blanklines") ? 2 : 0)
-                + 1
+                + table_header
                 + context.headers.size ()
                 + context.footnotes.size ();
 
@@ -660,11 +673,11 @@ int handleCustomReport (const std::string& report, std::string& outs)
           << table.rowCount ()
           << (table.rowCount () == 1 ? " task" : " tasks");
 
-      if (maxrows)
+      if (maxrows && maxrows < table.rowCount ())
         out << ", " << maxrows << " shown";
 
       if (maxlines && maxlines < table.rowCount ())
-        out << ", truncated to " << maxlines - 1 << " lines";
+        out << ", truncated to " << maxlines - table_header << " tasks";
 
       out << std::endl;
     }
@@ -714,7 +727,8 @@ void validReportColumns (const std::vector <std::string>& columns)
         *it != "description_only"     &&
         *it != "description"          &&
         *it != "wait"                 &&
-        *it != "depends")
+        *it != "depends"              &&
+        *it != "urgency")
       bad.push_back (*it);
 
   if (bad.size ())
