@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -51,7 +51,7 @@ static std::string findValue (
   std::string::size_type found = text.find (name);
   if (found != std::string::npos)
   {
-    std::string::size_type eol = text.find ("\n", found);
+    std::string::size_type eol = text.find ("\n", found + 1);
     if (eol != std::string::npos)
     {
       std::string value = text.substr (
@@ -73,7 +73,7 @@ static std::string findDate (
   std::string::size_type found = text.find (name);
   if (found != std::string::npos)
   {
-    std::string::size_type eol = text.find ("\n", found);
+    std::string::size_type eol = text.find ("\n", found + 1);
     if (eol != std::string::npos)
     {
       std::string value = trim (text.substr (
@@ -83,9 +83,7 @@ static std::string findDate (
       if (value != "")
       {
         Date dt (value, context.config.get ("dateformat"));
-        char epoch [16];
-        sprintf (epoch, "%d", (int)dt.toEpoch ());
-        return std::string (epoch);
+        return format ((int) dt.toEpoch ());
       }
     }
   }
@@ -112,22 +110,26 @@ static std::string formatDate (
 static std::string formatTask (Task task)
 {
   std::stringstream before;
-  before << "# The 'task edit <id>' command allows you to modify all aspects of a task\n"
-         << "# using a text editor.  What is shown below is a representation of the\n"
-         << "# task in all it's detail.  Modify what you wish, and if you save and\n"
-         << "# quit your editor, taskwarrior will read this file and try to make sense\n"
-         << "# of what changed, and apply those changes.  If you quit your editor\n"
-         << "# without saving or making any modifications, taskwarrior will do nothing.\n"
-         << "#\n"
-         << "# Lines that begin with # represent data you cannot change, like ID.\n"
-         << "# If you get too 'creative' with your editing, taskwarrior will dump you\n"
-         << "# back into the editor to try again.\n"
-         << "#\n"
-         << "# Should you find yourself in an endless Groundhog Day loop, editing and\n"
-         << "# editing the same file, just quit the editor without making any changes.\n"
-         << "# Taskwarrior will notice this and stop the editing.\n"
-         << "#\n"
-         << "# Name               Editable details\n"
+  bool verbose = context.config.getBoolean ("edit.verbose");
+
+  if (verbose)
+    before << "# The 'task edit <id>' command allows you to modify all aspects of a task\n"
+           << "# using a text editor.  What is shown below is a representation of the\n"
+           << "# task in all it's detail.  Modify what you wish, and if you save and\n"
+           << "# quit your editor, taskwarrior will read this file and try to make sense\n"
+           << "# of what changed, and apply those changes.  If you quit your editor\n"
+           << "# without saving or making any modifications, taskwarrior will do nothing.\n"
+           << "#\n"
+           << "# Lines that begin with # represent data you cannot change, like ID.\n"
+           << "# If you get too 'creative' with your editing, taskwarrior will dump you\n"
+           << "# back into the editor to try again.\n"
+           << "#\n"
+           << "# Should you find yourself in an endless Groundhog Day loop, editing and\n"
+           << "# editing the same file, just quit the editor without making any changes.\n"
+           << "# Taskwarrior will notice this and stop the editing.\n"
+           << "#\n";
+
+  before << "# Name               Editable details\n"
          << "# -----------------  ----------------------------------------------------\n"
          << "# ID:                " << task.id                                          << "\n"
          << "# UUID:              " << task.get ("uuid")                                << "\n"
@@ -141,10 +143,11 @@ static std::string formatTask (Task task)
   task.getTags (tags);
   std::string allTags;
   join (allTags, " ", tags);
-  before << "# Separate the tags with spaces, like this: tag1 tag2\n"
-         << "  Tags:              " << allTags                                          << "\n"
-         << "# The description field is allowed to wrap and use multiple lines.  Task\n"
-         << "# will combine them.\n"
+
+  if (verbose)
+    before << "# Separate the tags with spaces, like this: tag1 tag2\n";
+
+  before << "  Tags:              " << allTags                                          << "\n"
          << "  Description:       " << task.get ("description")                         << "\n"
          << "  Created:           " << formatDate (task, "entry")                       << "\n"
          << "  Started:           " << formatDate (task, "start")                       << "\n"
@@ -155,13 +158,17 @@ static std::string formatTask (Task task)
          << "  Wait until:        " << formatDate (task, "wait")                        << "\n"
          << "  Parent:            " << task.get ("parent")                              << "\n"
          << "  Foreground color:  " << task.get ("fg")                                  << "\n"
-         << "  Background color:  " << task.get ("bg")                                  << "\n"
-         << "# Annotations look like this: <date> -- <text> and there can be any number of them\n"
-         << "# ' -- ' is the separator between the date and text field. It should not be removed\n";
+         << "  Background color:  " << task.get ("bg")                                  << "\n";
+
+  if (verbose)
+    before << "# Annotations look like this: <date> -- <text> and there can be any number of them.\n"
+           << "# The ' -- ' separator between the date and text field should not be removed.\n"
+           << "# A \"blank slot\" for adding an annotation follows for your convenience.\n";
 
   std::vector <Att> annotations;
   task.getAnnotations (annotations);
-  foreach (anno, annotations)
+  std::vector <Att>::iterator anno;
+  for (anno = annotations.begin (); anno != annotations.end (); ++anno)
   {
     Date dt (::atoi (anno->name ().substr (11).c_str ()));
     before << "  Annotation:        " << dt.toString (context.config.get ("dateformat.annotation"))
@@ -176,8 +183,11 @@ static std::string formatTask (Task task)
   task.getDependencies (dependencies);
   std::string allDeps;
   join (allDeps, ",", dependencies);
-  before << "# Dependencies should be a comma-separated list of task IDs, with no spaces.\n"
-         << "  Dependencies:      " << allDeps << "\n";
+
+  if (verbose)
+    before << "# Dependencies should be a comma-separated list of task IDs, with no spaces.\n";
+
+  before << "  Dependencies:      " << allDeps << "\n";
 
   before << "# End\n";
   return before.str ();
@@ -187,7 +197,7 @@ static std::string formatTask (Task task)
 static void parseTask (Task& task, const std::string& after)
 {
   // project
-  std::string value = findValue (after, "Project:");
+  std::string value = findValue (after, "\n  Project:");
   if (task.get ("project") != value)
   {
     if (value != "")
@@ -203,7 +213,7 @@ static void parseTask (Task& task, const std::string& after)
   }
 
   // priority
-  value = findValue (after, "Priority:");
+  value = findValue (after, "\n  Priority:");
   if (task.get ("priority") != value)
   {
     if (value != "")
@@ -222,14 +232,14 @@ static void parseTask (Task& task, const std::string& after)
   }
 
   // tags
-  value = findValue (after, "Tags:");
+  value = findValue (after, "\n  Tags:");
   std::vector <std::string> tags;
   split (tags, value, ' ');
   task.remove ("tags");
   task.addTags (tags);
 
   // description.
-  value = findValue (after, "Description: ");
+  value = findValue (after, "\n  Description:");
   if (task.get ("description") != value)
   {
     if (value != "")
@@ -242,7 +252,7 @@ static void parseTask (Task& task, const std::string& after)
   }
 
   // entry
-  value = findDate (after, "Created:");
+  value = findDate (after, "\n  Created:");
   if (value != "")
   {
     Date edited (::atoi (value.c_str ()));
@@ -258,7 +268,7 @@ static void parseTask (Task& task, const std::string& after)
     throw std::string ("Cannot remove creation date.");
 
   // start
-  value = findDate (after, "Started:");
+  value = findDate (after, "\n  Started:");
   if (value != "")
   {
     Date edited (::atoi (value.c_str ()));
@@ -288,7 +298,7 @@ static void parseTask (Task& task, const std::string& after)
   }
 
   // end
-  value = findDate (after, "Ended:");
+  value = findDate (after, "\n  Ended:");
   if (value != "")
   {
     Date edited (::atoi (value.c_str ()));
@@ -316,7 +326,7 @@ static void parseTask (Task& task, const std::string& after)
   }
 
   // due
-  value = findDate (after, "Due:");
+  value = findDate (after, "\n  Due:");
   if (value != "")
   {
     Date edited (::atoi (value.c_str ()));
@@ -354,7 +364,7 @@ static void parseTask (Task& task, const std::string& after)
   }
 
   // until
-  value = findDate (after, "Until:");
+  value = findDate (after, "\n  Until:");
   if (value != "")
   {
     Date edited (::atoi (value.c_str ()));
@@ -384,7 +394,7 @@ static void parseTask (Task& task, const std::string& after)
   }
 
   // recur
-  value = findValue (after, "Recur:");
+  value = findValue (after, "\n  Recur:");
   if (value != task.get ("recur"))
   {
     if (value != "")
@@ -416,7 +426,7 @@ static void parseTask (Task& task, const std::string& after)
   }
 
   // wait
-  value = findDate (after, "Wait until:");
+  value = findDate (after, "\n  Wait until:");
   if (value != "")
   {
     Date edited (::atoi (value.c_str ()));
@@ -428,12 +438,14 @@ static void parseTask (Task& task, const std::string& after)
       {
         std::cout << "Wait date modified.\n";
         task.set ("wait", value);
+        task.setStatus (Task::waiting);
       }
     }
     else
     {
       std::cout << "Wait date modified.\n";
       task.set ("wait", value);
+      task.setStatus (Task::waiting);
     }
   }
   else
@@ -442,11 +454,12 @@ static void parseTask (Task& task, const std::string& after)
     {
       std::cout << "Wait date removed.\n";
       task.remove ("wait");
+      task.setStatus (Task::pending);
     }
   }
 
   // parent
-  value = findValue (after, "Parent:");
+  value = findValue (after, "\n  Parent:");
   if (value != task.get ("parent"))
   {
     if (value != "")
@@ -462,7 +475,7 @@ static void parseTask (Task& task, const std::string& after)
   }
 
   // fg
-  value = findValue (after, "Foreground color:");
+  value = findValue (after, "\n  Foreground color:");
   if (value != task.get ("fg"))
   {
     if (value != "")
@@ -478,7 +491,7 @@ static void parseTask (Task& task, const std::string& after)
   }
 
   // bg
-  value = findValue (after, "Background color:");
+  value = findValue (after, "\n  Background color:");
   if (value != task.get ("bg"))
   {
     if (value != "")
@@ -496,11 +509,11 @@ static void parseTask (Task& task, const std::string& after)
   // Annotations
   std::vector <Att> annotations;
   std::string::size_type found = 0;
-  while ((found = after.find ("Annotation:", found)) != std::string::npos)
+  while ((found = after.find ("\n  Annotation:", found)) != std::string::npos)
   {
-    found += 11;
+    found += 14;  // Length of "\n  Annotation:".
 
-    std::string::size_type eol = after.find ("\n", found);
+    std::string::size_type eol = after.find ("\n", found + 1);
     if (eol != std::string::npos)
     {
       std::string value = trim (after.substr (
@@ -528,12 +541,13 @@ static void parseTask (Task& task, const std::string& after)
   task.setAnnotations (annotations);
 
   // Dependencies
-  value = findValue (after, "Dependencies:");
+  value = findValue (after, "\n  Dependencies:");
   std::vector <std::string> dependencies;
   split (dependencies, value, ",");
 
   task.remove ("depends");
-  foreach (dep, dependencies)
+  std::vector <std::string>::iterator dep;
+  for (dep = dependencies.begin (); dep != dependencies.end (); ++dep)
   {
     int id = atoi (dep->c_str ());
     if (id)
@@ -542,7 +556,7 @@ static void parseTask (Task& task, const std::string& after)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-void editFile (Task& task)
+bool editFile (Task& task)
 {
   // Check for file permissions.
   Directory location (context.config.get ("data.location"));
@@ -573,6 +587,8 @@ void editFile (Task& task)
   editor += "\"" + file.str () + "\"";
 
 ARE_THESE_REALLY_HARMFUL:
+  bool changes = false; // No changes made.
+
   // Launch the editor.
   std::cout << "Launching '" << editor << "' now...\n";
   if (-1 == system (editor.c_str ()))
@@ -614,12 +630,18 @@ ARE_THESE_REALLY_HARMFUL:
       if (confirm ("Taskwarrior couldn't handle your edits.  Would you like to try again?"))
         goto ARE_THESE_REALLY_HARMFUL;
     }
+    else
+      changes = true;
   }
   else
+  {
     std::cout << "No edits were detected.\n";
+    changes = false;
+  }
 
   // Cleanup.
   File::remove (file.str ());
+  return changes;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -644,35 +666,10 @@ int handleEdit (std::string& outs)
     std::vector <Task> all = tasks;
     context.filter.applySequence (tasks, context.sequence);
 
-    foreach (task, tasks)
-    {
-      editFile (*task);
-      context.tdb.update (*task);
-/*
-      TODO Figure out what this is.  I can't remember, but don't want to remove
-           it until I do.
-
-      foreach (other, all)
-      {
-        if (other->id != task.id) // Don't edit the same task again.
-        {
-          if (task.has ("parent") && 
-          if (other is parent of task)
-          {
-            // Transfer everything but mask, imask, uuid, parent.
-          }
-          else if (task is parent of other)
-          {
-            // Transfer everything but mask, imask, uuid, parent.
-          }
-          else if (task and other are siblings)
-          {
-            // Transfer everything but mask, imask, uuid, parent.
-          }
-        }
-      }
-*/
-    }
+    std::vector <Task>::iterator task;
+    for (task = tasks.begin (); task != tasks.end (); ++task)
+      if (editFile (*task))
+        context.tdb.update (*task);
 
     context.tdb.commit ();
     context.tdb.unlock ();
