@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -192,6 +192,13 @@ static void decorateTask (Task& task)
       defaultPriority != "" &&
       Att::validNameValue ("priority", "", defaultPriority))
     task.set ("priority", defaultPriority);
+
+  // Override with default.due, if not specified.
+  std::string defaultDue = context.config.get ("default.due");
+  if (!task.has ("due") &&
+      defaultDue != "" &&
+      Att::validNameValue ("due", "", defaultDue))
+    task.set ("due", defaultDue);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1260,16 +1267,16 @@ int handleImport (std::string& outs)
 
   if (context.hooks.trigger ("pre-import-command"))
   {
-    std::stringstream out;		
-		
+    std::stringstream out;
+
 		// Use the description as a file name.
     std::string file = trim (context.task.get ("description"));
-		
-#if FEATURE_URL > 0
+
+#ifdef FEATURE_URL
     std::string tmpfile = "";
     Uri uri (file);
     uri.parse ();
-    
+
     Transport* transport;
     if ((transport = Transport::getTransport (uri)) != NULL )
     {
@@ -1281,7 +1288,7 @@ int handleImport (std::string& outs)
       file = tmpfile;
     }
 #endif
-    
+
     if (file.length () > 0)
     {
       // Load the file.
@@ -1337,11 +1344,9 @@ int handleImport (std::string& outs)
       case not_a_clue:    /* to stop the compiler from complaining. */ break;
       }
 
-#if FEATURE_URL > 0
+#ifdef FEATURE_URL
       if (tmpfile != "")
-      {
         remove (tmpfile.c_str ());
-      }
 #endif
     }
     else
