@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -33,6 +33,8 @@
 #define FEATURE_NCURSES_COLS 1   // Shortcut that avoids WINDOW.
 #define FEATURE_URL          1   // URL support for import.
                                  // (always enabled for merge/push/pull)
+//#define FEATURE_REGEX        1   // Enables regexes for attribute modifiers,
+//                                 // subst, general search.
 
 #include <string>
 #include <vector>
@@ -68,6 +70,9 @@ int handleCompletionIDs (std::string&);
 int handleCompletionConfig (std::string&);
 int handleCompletionVersion (std::string&);
 int handleUrgency (std::string&);
+int handleQuery (std::string&);
+int handleZshCompletionCommands (std::string&);
+int handleZshCompletionIDs (std::string&);
 int handleVersion (std::string&);
 int handleConfig (std::string&);
 int handleShow (std::string&);
@@ -78,6 +83,7 @@ int handleColor (std::string&);
 int handleAnnotate (std::string&);
 int handleDenotate (std::string&);
 int handleDuplicate (std::string&);
+int handleCount (std::string&);
 void handleUndo ();
 void handleMerge (std::string&);
 void handlePush (std::string&);
@@ -103,10 +109,6 @@ int shortUsage (std::string&);
 int longUsage (std::string&);
 int handleInfo (std::string&);
 int handleReportSummary (std::string&);
-int handleReportHistoryMonthly (std::string&);
-int handleReportHistoryAnnual (std::string&);
-int handleReportGHistoryMonthly (std::string&);
-int handleReportGHistoryAnnual (std::string&);
 int handleReportCalendar (std::string&);
 int handleReportStats (std::string&);
 int handleReportTimesheet (std::string&);
@@ -116,6 +118,17 @@ std::string getDueDate (Task&, const std::string&);
 std::string onProjectChange (Task&, bool scope = true);
 std::string onProjectChange (Task&, Task&);
 
+// burndown.cpp
+int handleReportBurndownDaily (std::string&);
+int handleReportBurndownWeekly (std::string&);
+int handleReportBurndownMonthly (std::string&);
+
+// history.cpp
+int handleReportHistoryMonthly (std::string&);
+int handleReportHistoryAnnual (std::string&);
+int handleReportGHistoryMonthly (std::string&);
+int handleReportGHistoryAnnual (std::string&);
+
 // custom.cpp
 int handleCustomReport (const std::string&, std::string&);
 void validReportColumns (const std::vector <std::string>&);
