@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -44,10 +44,6 @@
 #include "util.h"
 #include "main.h"
 
-#ifdef HAVE_LIBNCURSES
-#include <ncurses.h>
-#endif
-
 // Global context for use by all.
 extern Context context;
 
@@ -100,6 +96,7 @@ void handleRecurrence ()
           Task rec (*t);                         // Clone the parent.
           rec.set ("uuid", uuid ());             // New UUID.
           rec.set ("parent", t->get ("uuid"));   // Remember mom.
+          rec.setEntry ();                       // New entry date.
 
           char dueDate[16];
           sprintf (dueDate, "%u", (unsigned int) d->toEpoch ());
@@ -222,7 +219,7 @@ Date getNextRecurrence (Date& current, std::string& period)
     return Date (m, d, y);
   }
 
-  if (period == "weekdays")
+  else if (period == "weekdays")
   {
     int dow = current.dayOfWeek ();
     int days;
@@ -234,7 +231,7 @@ Date getNextRecurrence (Date& current, std::string& period)
     return current + (days * 86400);
   }
 
-  if (isdigit (period[0]) && period[period.length () - 1] == 'm')
+  else if (isdigit (period[0]) && period[period.length () - 1] == 'm')
   {
     std::string numeric = period.substr (0, period.length () - 1);
     int increment = atoi (numeric.c_str ());
@@ -337,9 +334,10 @@ Date getNextRecurrence (Date& current, std::string& period)
   }
 
   // If the period is an 'easy' one, add it to current, and we're done.
+  // If it throws an error, the duration was not recognized.
   int secs = 0;
-  try { Duration du (period); secs = du; }
-  catch (...) { secs = 0; }
+  Duration du (period);
+  secs = du;
 
   return current + secs;
 }
