@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -37,19 +37,15 @@
 #include <pwd.h>
 #include <time.h>
 
-#include "Context.h"
-#include "Directory.h"
-#include "File.h"
-#include "Date.h"
-#include "Duration.h"
-#include "Table.h"
-#include "text.h"
-#include "util.h"
-#include "main.h"
-
-#ifdef HAVE_LIBNCURSES
-#include <ncurses.h>
-#endif
+#include <Context.h>
+#include <Directory.h>
+#include <File.h>
+#include <Date.h>
+#include <Duration.h>
+#include <Table.h>
+#include <text.h>
+#include <util.h>
+#include <main.h>
 
 static void countTasks (const std::vector <Task>&, const std::string&, const std::string&, int&, int&);
 
@@ -187,6 +183,18 @@ int shortUsage (std::string& outs)
   table.addCell (row, 1, "task ghistory.annual");
   table.addCell (row, 2, "Shows a graphical report of task history, by year.");
 
+  row = table.addRow ();
+  table.addCell (row, 1, "task burndown.daily");
+  table.addCell (row, 2, "Shows a graphical burndown chart, by day.");
+
+  row = table.addRow ();
+  table.addCell (row, 1, "task burndown.weekly");
+  table.addCell (row, 2, "Shows a graphical burndown chart, by week.");
+
+  row = table.addRow ();
+  table.addCell (row, 1, "task burndown.monthly");
+  table.addCell (row, 2, "Shows a graphical burndown chart, by month.");
+
   row = table.addRow ();
   table.addCell (row, 1, "task calendar [due|month year|year]");
   table.addCell (row, 2, "Shows a calendar, with due tasks marked.");
@@ -232,6 +240,10 @@ int shortUsage (std::string& outs)
   table.addCell (row, 2, "Displays all possible colors, a named sample, or a "
                          "legend containing all currently defined colors.");
 
+  row = table.addRow ();
+  table.addCell (row, 1, "task count [filter]");
+  table.addCell (row, 2, "Shows only the number of matching tasks.");
+
   row = table.addRow ();
   table.addCell (row, 1, "task version");
   table.addCell (row, 2, "Shows the task version number.");
@@ -323,7 +335,7 @@ int longUsage (std::string& outs)
         << "  any"                                                              << "\n"
         << "  is         (synonym equals)"                                      << "\n"
         << "  isnt       (synonym not)"                                         << "\n"
-        << "  has        (synonym contain)"                                     << "\n"
+        << "  has        (synonym contains)"                                    << "\n"
         << "  hasnt"                                                            << "\n"
         << "  startswith (synonym left)"                                        << "\n"
         << "  endswith   (synonym right)"                                       << "\n"
@@ -380,6 +392,15 @@ int handleInfo (std::string& outs)
     // Filter sequence.
     context.filter.applySequence (tasks, context.sequence);
 
+    // Read the undo file.
+    std::vector <std::string> undo;
+    if (context.config.getBoolean ("journal.info"))
+    {
+      Directory location (context.config.get ("data.location"));
+      std::string undoFile = location.data + "/undo.data";
+      File::read (undoFile, undo);
+    }
+
     // Find the task.
     std::stringstream out;
     foreach (task, tasks)
@@ -410,12 +431,9 @@ int handleInfo (std::string& outs)
       context.hooks.trigger ("pre-display", *task);
 
       // id
-      char svalue[12];
-      std::string value;
       int row = table.addRow ();
       table.addCell (row, 0, "ID");
-      sprintf (svalue, "%d", (int) task->id);
-      value = svalue;
+      std::string value = format (task->id);
       context.hooks.trigger ("format-id", "id", value);
       table.addCell (row, 1, value);
 
@@ -598,7 +616,7 @@ int handleInfo (std::string& outs)
       // uuid
       row = table.addRow ();
       table.addCell (row, 0, "UUID");
-      value = task->get ("uuid");
+      std::string uuid = value = task->get ("uuid");
       context.hooks.trigger ("format-uuid", "uuid", value);
       table.addCell (row, 1, value);
 
@@ -643,20 +661,117 @@ int handleInfo (std::string& outs)
       //table.addCell (row, 0, "Urgency");
       //table.addCell (row, 1, task->urgency ());
 
+      // Create a second table, containing undo log change details.
+      Table journal;
+
       // If an alternating row color is specified, notify the table.
       if (context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor"))
       {
         Color alternate (context.config.get ("color.alternate"));
         if (alternate.nontrivial ())
+        {
           table.setTableAlternateColor (alternate);
+          journal.setTableAlternateColor (alternate);
+        }
+      }
+
+      journal.setTableWidth (context.getWidth ());
+      journal.setDateFormat (context.config.get ("dateformat"));
+
+      journal.addColumn ("Date");
+      journal.addColumn ("Modification");
+
+      if ((context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor")) &&
+          context.config.getBoolean ("fontunderline"))
+      {
+        journal.setColumnUnderline (0);
+        journal.setColumnUnderline (1);
+      }
+      else
+        journal.setTableDashedUnderline ();
+
+      journal.setColumnWidth (0, Table::minimum);
+      journal.setColumnWidth (1, Table::flexible);
+
+      journal.setColumnJustification (0, Table::left);
+      journal.setColumnJustification (1, Table::left);
+
+      if (context.config.getBoolean ("journal.info") &&
+          undo.size () > 3)
+      {
+        // Scan the undo data for entries matching this task.
+        std::string when;
+        std::string previous;
+        std::string current;
+        unsigned int i = 0;
+        long total_time = 0;
+        while (i < undo.size ())
+        {
+          when = undo[i++];
+          previous = "";
+          if (undo[i].substr (0, 3) == "old")
+            previous = undo[i++];
+
+          current = undo[i++];
+          i++; // Separator
+
+          if (current.find ("uuid:\"" + uuid) != std::string::npos)
+          {
+            if (previous != "")
+            {
+              int row = journal.addRow ();
+
+              Date timestamp (atoi (when.substr (5).c_str ()));
+              journal.addCell (row, 0, timestamp.toString (context.config.get ("dateformat")));
+
+              Task before (previous.substr (4));
+              Task after (current.substr (4));
+              journal.addCell (row, 1, taskInfoDifferences (before, after));
+
+              // calculate the total active time
+              if (before.get ("start") == ""
+                && after.get ("start") != "")
+              {
+                // task started
+                total_time -= timestamp.toEpoch ();
+              }
+              else if (before.get ("start") != ""
+                     && after.get ("start") == "")
+              {
+                // task stopped
+                total_time += timestamp.toEpoch ();
+              }
+            }
+          }
+        }
+
+        // add now() if task is still active
+        if (total_time < 0)
+          total_time += Date ().toEpoch ();
+
+        // print total active time
+        if (total_time > 0)
+        {
+          row = journal.addRow ();
+          journal.addCell (row, 0, "Total active time");
+          journal.addCell (row, 1, Duration (total_time).format ());
+
+          if (context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor"))
+            journal.setCellColor (row, 1, Color ("bold"));
+        }
       }
 
       out << optionalBlankLine ()
           << table.render ()
           << "\n";
+
+      if (journal.rowCount () > 0)
+        out << journal.render ()
+            << "\n";
     }
 
-    if (! tasks.size ()) {
+    if (! tasks.size ())
+    {
       out << "No matches.\n";
       rc = 1;
     }
@@ -819,787 +934,6 @@ int handleReportSummary (std::string& outs)
   return rc;
 }
 
-////////////////////////////////////////////////////////////////////////////////
-// Year Month    Added Completed Deleted
-// 2006 November    87        63      14
-//      December    21         6       1
-// 2007 January      3        12       0
-time_t monthlyEpoch (const std::string& date)
-{
-  // Convert any date in epoch form to m/d/y, then convert back
-  // to epoch form for the date m/1/y.
-  if (date.length ())
-  {
-    Date d1 (atoi (date.c_str ()));
-    int m, d, y;
-    d1.toMDY (m, d, y);
-    Date d2 (m, 1, y);
-    time_t epoch;
-    d2.toEpoch (epoch);
-    return epoch;
- }
-
-  return 0;
-}
-
-time_t yearlyEpoch (const std::string& date)
-{
-  // Convert any date in epoch form to m/d/y, then convert back
-  // to epoch form for the date 1/1/y.
-  if (date.length ())
-  {
-    Date d1 (atoi (date.c_str ()));
-    int m, d, y;
-    d1.toMDY (m, d, y);
-    Date d2 (1, 1, y);
-    time_t epoch;
-    d2.toEpoch (epoch);
-    return epoch;
- }
-
-  return 0;
-}
-
-int handleReportHistoryMonthly (std::string& outs)
-{
-  int rc = 0;
-
-  if (context.hooks.trigger ("pre-history-command"))
-  {
-    std::map <time_t, int> groups;          // Represents any month with data
-    std::map <time_t, int> addedGroup;      // Additions by month
-    std::map <time_t, int> completedGroup;  // Completions by month
-    std::map <time_t, int> deletedGroup;    // Deletions by month
-
-    // Scan the pending tasks.
-    std::vector <Task> tasks;
-    context.tdb.lock (context.config.getBoolean ("locking"));
-    handleRecurrence ();
-    context.tdb.load (tasks, context.filter);
-    context.tdb.commit ();
-    context.tdb.unlock ();
-
-    foreach (task, tasks)
-    {
-      time_t epoch = monthlyEpoch (task->get ("entry"));
-      groups[epoch] = 0;
-
-      // Every task has an entry date.
-      if (addedGroup.find (epoch) != addedGroup.end ())
-        addedGroup[epoch] = addedGroup[epoch] + 1;
-      else
-        addedGroup[epoch] = 1;
-
-      // All deleted tasks have an end date.
-      if (task->getStatus () == Task::deleted)
-      {
-        epoch = monthlyEpoch (task->get ("end"));
-        groups[epoch] = 0;
-
-        if (deletedGroup.find (epoch) != deletedGroup.end ())
-          deletedGroup[epoch] = deletedGroup[epoch] + 1;
-        else
-          deletedGroup[epoch] = 1;
-      }
-
-      // All completed tasks have an end date.
-      else if (task->getStatus () == Task::completed)
-      {
-        epoch = monthlyEpoch (task->get ("end"));
-        groups[epoch] = 0;
-
-        if (completedGroup.find (epoch) != completedGroup.end ())
-          completedGroup[epoch] = completedGroup[epoch] + 1;
-        else
-          completedGroup[epoch] = 1;
-      }
-    }
-
-    // Now build the table.
-    Table table;
-    table.setDateFormat (context.config.get ("dateformat"));
-    table.addColumn ("Year");
-    table.addColumn ("Month");
-    table.addColumn ("Added");
-    table.addColumn ("Completed");
-    table.addColumn ("Deleted");
-    table.addColumn ("Net");
-
-    if ((context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor")) &&
-        context.config.getBoolean ("fontunderline"))
-    {
-      table.setColumnUnderline (0);
-      table.setColumnUnderline (1);
-      table.setColumnUnderline (2);
-      table.setColumnUnderline (3);
-      table.setColumnUnderline (4);
-      table.setColumnUnderline (5);
-    }
-    else
-      table.setTableDashedUnderline ();
-
-    table.setColumnJustification (2, Table::right);
-    table.setColumnJustification (3, Table::right);
-    table.setColumnJustification (4, Table::right);
-    table.setColumnJustification (5, Table::right);
-
-    int totalAdded     = 0;
-    int totalCompleted = 0;
-    int totalDeleted   = 0;
-
-    int priorYear = 0;
-    int row = 0;
-    foreach (i, groups)
-    {
-      row = table.addRow ();
-
-      totalAdded     += addedGroup     [i->first];
-      totalCompleted += completedGroup [i->first];
-      totalDeleted   += deletedGroup   [i->first];
-
-      Date dt (i->first);
-      int m, d, y;
-      dt.toMDY (m, d, y);
-
-      if (y != priorYear)
-      {
-        table.addCell (row, 0, y);
-        priorYear = y;
-      }
-      table.addCell (row, 1, Date::monthName(m));
-
-      int net = 0;
-
-      if (addedGroup.find (i->first) != addedGroup.end ())
-      {
-        table.addCell (row, 2, addedGroup[i->first]);
-        net +=addedGroup[i->first];
-      }
-
-      if (completedGroup.find (i->first) != completedGroup.end ())
-      {
-        table.addCell (row, 3, completedGroup[i->first]);
-        net -= completedGroup[i->first];
-      }
-
-      if (deletedGroup.find (i->first) != deletedGroup.end ())
-      {
-        table.addCell (row, 4, deletedGroup[i->first]);
-        net -= deletedGroup[i->first];
-      }
-
-      table.addCell (row, 5, net);
-      if ((context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor")) && net)
-        table.setCellColor (row, 5, net > 0 ? Color (Color::red) :
-                                              Color (Color::green));
-    }
-
-    if (table.rowCount ())
-    {
-      table.addRow ();
-      row = table.addRow ();
-
-      table.addCell (row, 1, "Average");
-      if (context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor"))
-        table.setRowColor (row, Color (Color::nocolor, Color::nocolor, false, true, false));
-      table.addCell (row, 2, totalAdded     / (table.rowCount () - 2));
-      table.addCell (row, 3, totalCompleted / (table.rowCount () - 2));
-      table.addCell (row, 4, totalDeleted   / (table.rowCount () - 2));
-      table.addCell (row, 5, (totalAdded - totalCompleted - totalDeleted) / (table.rowCount () - 2));
-    }
-
-    std::stringstream out;
-    if (table.rowCount ())
-      out << optionalBlankLine ()
-          << table.render ()
-          << "\n";
-    else
-    {
-      out << "No tasks.\n";
-      rc = 1;
-    }
-
-    outs = out.str ();
-    context.hooks.trigger ("post-history-command");
-  }
-
-  return rc;
-}
-
-////////////////////////////////////////////////////////////////////////////////
-int handleReportHistoryAnnual (std::string& outs)
-{
-  int rc = 0;
-
-  if (context.hooks.trigger ("pre-history-command"))
-  {
-    std::map <time_t, int> groups;          // Represents any month with data
-    std::map <time_t, int> addedGroup;      // Additions by month
-    std::map <time_t, int> completedGroup;  // Completions by month
-    std::map <time_t, int> deletedGroup;    // Deletions by month
-
-    // Scan the pending tasks.
-    std::vector <Task> tasks;
-    context.tdb.lock (context.config.getBoolean ("locking"));
-    handleRecurrence ();
-    context.tdb.load (tasks, context.filter);
-    context.tdb.commit ();
-    context.tdb.unlock ();
-
-    foreach (task, tasks)
-    {
-      time_t epoch = yearlyEpoch (task->get ("entry"));
-      groups[epoch] = 0;
-
-      // Every task has an entry date.
-      if (addedGroup.find (epoch) != addedGroup.end ())
-        addedGroup[epoch] = addedGroup[epoch] + 1;
-      else
-        addedGroup[epoch] = 1;
-
-      // All deleted tasks have an end date.
-      if (task->getStatus () == Task::deleted)
-      {
-        epoch = yearlyEpoch (task->get ("end"));
-        groups[epoch] = 0;
-
-        if (deletedGroup.find (epoch) != deletedGroup.end ())
-          deletedGroup[epoch] = deletedGroup[epoch] + 1;
-        else
-          deletedGroup[epoch] = 1;
-      }
-
-      // All completed tasks have an end date.
-      else if (task->getStatus () == Task::completed)
-      {
-        epoch = yearlyEpoch (task->get ("end"));
-        groups[epoch] = 0;
-
-        if (completedGroup.find (epoch) != completedGroup.end ())
-          completedGroup[epoch] = completedGroup[epoch] + 1;
-        else
-          completedGroup[epoch] = 1;
-      }
-    }
-
-    // Now build the table.
-    Table table;
-    table.setDateFormat (context.config.get ("dateformat"));
-    table.addColumn ("Year");
-    table.addColumn ("Added");
-    table.addColumn ("Completed");
-    table.addColumn ("Deleted");
-    table.addColumn ("Net");
-
-    if ((context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor")) &&
-        context.config.getBoolean ("fontunderline"))
-    {
-      table.setColumnUnderline (0);
-      table.setColumnUnderline (1);
-      table.setColumnUnderline (2);
-      table.setColumnUnderline (3);
-      table.setColumnUnderline (4);
-    }
-    else
-      table.setTableDashedUnderline ();
-
-    table.setColumnJustification (1, Table::right);
-    table.setColumnJustification (2, Table::right);
-    table.setColumnJustification (3, Table::right);
-    table.setColumnJustification (4, Table::right);
-
-    int totalAdded     = 0;
-    int totalCompleted = 0;
-    int totalDeleted   = 0;
-
-    int priorYear = 0;
-    int row = 0;
-    foreach (i, groups)
-    {
-      row = table.addRow ();
-
-      totalAdded     += addedGroup     [i->first];
-      totalCompleted += completedGroup [i->first];
-      totalDeleted   += deletedGroup   [i->first];
-
-      Date dt (i->first);
-      int m, d, y;
-      dt.toMDY (m, d, y);
-
-      if (y != priorYear)
-      {
-        table.addCell (row, 0, y);
-        priorYear = y;
-      }
-
-      int net = 0;
-
-      if (addedGroup.find (i->first) != addedGroup.end ())
-      {
-        table.addCell (row, 1, addedGroup[i->first]);
-        net +=addedGroup[i->first];
-      }
-
-      if (completedGroup.find (i->first) != completedGroup.end ())
-      {
-        table.addCell (row, 2, completedGroup[i->first]);
-        net -= completedGroup[i->first];
-      }
-
-      if (deletedGroup.find (i->first) != deletedGroup.end ())
-      {
-        table.addCell (row, 3, deletedGroup[i->first]);
-        net -= deletedGroup[i->first];
-      }
-
-      table.addCell (row, 4, net);
-      if ((context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor")) && net)
-        table.setCellColor (row, 4, net > 0 ? Color (Color::red) :
-                                              Color (Color::green));
-    }
-
-    if (table.rowCount ())
-    {
-      table.addRow ();
-      row = table.addRow ();
-
-      table.addCell (row, 0, "Average");
-      if (context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor"))
-        table.setRowColor (row, Color (Color::nocolor, Color::nocolor, false, true, false));
-      table.addCell (row, 1, totalAdded     / (table.rowCount () - 2));
-      table.addCell (row, 2, totalCompleted / (table.rowCount () - 2));
-      table.addCell (row, 3, totalDeleted   / (table.rowCount () - 2));
-      table.addCell (row, 4, (totalAdded - totalCompleted - totalDeleted) / (table.rowCount () - 2));
-    }
-
-    std::stringstream out;
-    if (table.rowCount ())
-      out << optionalBlankLine ()
-          << table.render ()
-          << "\n";
-    else
-    {
-      out << "No tasks.\n";
-      rc = 1;
-    }
-
-    outs = out.str ();
-    context.hooks.trigger ("post-history-command");
-  }
-
-  return rc;
-}
-
-////////////////////////////////////////////////////////////////////////////////
-int handleReportGHistoryMonthly (std::string& outs)
-{
-  int rc = 0;
-
-  if (context.hooks.trigger ("pre-ghistory-command"))
-  {
-    std::map <time_t, int> groups;          // Represents any month with data
-    std::map <time_t, int> addedGroup;      // Additions by month
-    std::map <time_t, int> completedGroup;  // Completions by month
-    std::map <time_t, int> deletedGroup;    // Deletions by month
-
-    // Scan the pending tasks.
-    std::vector <Task> tasks;
-    context.tdb.lock (context.config.getBoolean ("locking"));
-    handleRecurrence ();
-    context.tdb.load (tasks, context.filter);
-    context.tdb.commit ();
-    context.tdb.unlock ();
-
-    foreach (task, tasks)
-    {
-      time_t epoch = monthlyEpoch (task->get ("entry"));
-      groups[epoch] = 0;
-
-      // Every task has an entry date.
-      if (addedGroup.find (epoch) != addedGroup.end ())
-        addedGroup[epoch] = addedGroup[epoch] + 1;
-      else
-        addedGroup[epoch] = 1;
-
-      // All deleted tasks have an end date.
-      if (task->getStatus () == Task::deleted)
-      {
-        epoch = monthlyEpoch (task->get ("end"));
-        groups[epoch] = 0;
-
-        if (deletedGroup.find (epoch) != deletedGroup.end ())
-          deletedGroup[epoch] = deletedGroup[epoch] + 1;
-        else
-          deletedGroup[epoch] = 1;
-      }
-
-      // All completed tasks have an end date.
-      else if (task->getStatus () == Task::completed)
-      {
-        epoch = monthlyEpoch (task->get ("end"));
-        groups[epoch] = 0;
-
-        if (completedGroup.find (epoch) != completedGroup.end ())
-          completedGroup[epoch] = completedGroup[epoch] + 1;
-        else
-          completedGroup[epoch] = 1;
-      }
-    }
-
-    int widthOfBar = context.getWidth () - 15;   // 15 == strlen ("2008 September ")
-
-    // Now build the table.
-    Table table;
-    table.setDateFormat (context.config.get ("dateformat"));
-    table.addColumn ("Year");
-    table.addColumn ("Month");
-    table.addColumn ("Number Added/Completed/Deleted");
-
-    if ((context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor")) &&
-        context.config.getBoolean ("fontunderline"))
-    {
-      table.setColumnUnderline (0);
-      table.setColumnUnderline (1);
-    }
-    else
-      table.setTableDashedUnderline ();
-
-    Color color_add    (context.config.get ("color.history.add"));
-    Color color_done   (context.config.get ("color.history.done"));
-    Color color_delete (context.config.get ("color.history.delete"));
-
-    // Determine the longest line, and the longest "added" line.
-    int maxAddedLine = 0;
-    int maxRemovedLine = 0;
-    foreach (i, groups)
-    {
-      if (completedGroup[i->first] + deletedGroup[i->first] > maxRemovedLine)
-        maxRemovedLine = completedGroup[i->first] + deletedGroup[i->first];
-
-      if (addedGroup[i->first] > maxAddedLine)
-        maxAddedLine = addedGroup[i->first];
-    }
-
-    int maxLine = maxAddedLine + maxRemovedLine;
-    if (maxLine > 0)
-    {
-      unsigned int leftOffset = (widthOfBar * maxAddedLine) / maxLine;
-
-      int totalAdded     = 0;
-      int totalCompleted = 0;
-      int totalDeleted   = 0;
-
-      int priorYear = 0;
-      int row = 0;
-      foreach (i, groups)
-      {
-        row = table.addRow ();
-
-        totalAdded     += addedGroup[i->first];
-        totalCompleted += completedGroup[i->first];
-        totalDeleted   += deletedGroup[i->first];
-
-        Date dt (i->first);
-        int m, d, y;
-        dt.toMDY (m, d, y);
-
-        if (y != priorYear)
-        {
-          table.addCell (row, 0, y);
-          priorYear = y;
-        }
-        table.addCell (row, 1, Date::monthName(m));
-
-        unsigned int addedBar     = (widthOfBar *     addedGroup[i->first]) / maxLine;
-        unsigned int completedBar = (widthOfBar * completedGroup[i->first]) / maxLine;
-        unsigned int deletedBar   = (widthOfBar *   deletedGroup[i->first]) / maxLine;
-
-        std::string bar = "";
-        if (context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor"))
-        {
-          char number[24];
-          std::string aBar = "";
-          if (addedGroup[i->first])
-          {
-            sprintf (number, "%d", addedGroup[i->first]);
-            aBar = number;
-            while (aBar.length () < addedBar)
-              aBar = " " + aBar;
-          }
-
-          std::string cBar = "";
-          if (completedGroup[i->first])
-          {
-            sprintf (number, "%d", completedGroup[i->first]);
-            cBar = number;
-            while (cBar.length () < completedBar)
-              cBar = " " + cBar;
-          }
-
-          std::string dBar = "";
-          if (deletedGroup[i->first])
-          {
-            sprintf (number, "%d", deletedGroup[i->first]);
-            dBar = number;
-            while (dBar.length () < deletedBar)
-              dBar = " " + dBar;
-          }
-
-          bar += std::string (leftOffset - aBar.length (), ' ');
-
-          bar += color_add.colorize    (aBar);
-          bar += color_done.colorize   (cBar);
-          bar += color_delete.colorize (dBar);
-        }
-        else
-        {
-          std::string aBar = ""; while (aBar.length () < addedBar)     aBar += "+";
-          std::string cBar = ""; while (cBar.length () < completedBar) cBar += "X";
-          std::string dBar = ""; while (dBar.length () < deletedBar)   dBar += "-";
-
-          bar += std::string (leftOffset - aBar.length (), ' ');
-          bar += aBar + cBar + dBar;
-        }
-
-        table.addCell (row, 2, bar);
-      }
-    }
-
-    std::stringstream out;
-    if (table.rowCount ())
-    {
-      out << optionalBlankLine ()
-          << table.render ()
-          << "\n";
-
-      if (context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor"))
-        out << "Legend: "
-            << color_add.colorize ("added")
-            << ", "
-            << color_done.colorize ("completed")
-            << ", "
-            << color_delete.colorize ("deleted")
-            << optionalBlankLine ()
-            << "\n";
-      else
-        out << "Legend: + added, X completed, - deleted\n";
-    }
-    else
-    {
-      out << "No tasks.\n";
-      rc = 1;
-    }
-
-    outs = out.str ();
-    context.hooks.trigger ("post-ghistory-command");
-  }
-
-  return rc;
-}
-
-////////////////////////////////////////////////////////////////////////////////
-int handleReportGHistoryAnnual (std::string& outs)
-{
-  int rc = 0;
-
-  if (context.hooks.trigger ("pre-ghistory-command"))
-  {
-    std::map <time_t, int> groups;          // Represents any month with data
-    std::map <time_t, int> addedGroup;      // Additions by month
-    std::map <time_t, int> completedGroup;  // Completions by month
-    std::map <time_t, int> deletedGroup;    // Deletions by month
-
-    // Scan the pending tasks.
-    std::vector <Task> tasks;
-    context.tdb.lock (context.config.getBoolean ("locking"));
-    handleRecurrence ();
-    context.tdb.load (tasks, context.filter);
-    context.tdb.commit ();
-    context.tdb.unlock ();
-
-    foreach (task, tasks)
-    {
-      time_t epoch = yearlyEpoch (task->get ("entry"));
-      groups[epoch] = 0;
-
-      // Every task has an entry date.
-      if (addedGroup.find (epoch) != addedGroup.end ())
-        addedGroup[epoch] = addedGroup[epoch] + 1;
-      else
-        addedGroup[epoch] = 1;
-
-      // All deleted tasks have an end date.
-      if (task->getStatus () == Task::deleted)
-      {
-        epoch = yearlyEpoch (task->get ("end"));
-        groups[epoch] = 0;
-
-        if (deletedGroup.find (epoch) != deletedGroup.end ())
-          deletedGroup[epoch] = deletedGroup[epoch] + 1;
-        else
-          deletedGroup[epoch] = 1;
-      }
-
-      // All completed tasks have an end date.
-      else if (task->getStatus () == Task::completed)
-      {
-        epoch = yearlyEpoch (task->get ("end"));
-        groups[epoch] = 0;
-
-        if (completedGroup.find (epoch) != completedGroup.end ())
-          completedGroup[epoch] = completedGroup[epoch] + 1;
-        else
-          completedGroup[epoch] = 1;
-      }
-    }
-
-    int widthOfBar = context.getWidth () - 5;   // 5 == strlen ("2008 ")
-
-    // Now build the table.
-    Table table;
-    table.setDateFormat (context.config.get ("dateformat"));
-    table.addColumn ("Year");
-    table.addColumn ("Number Added/Completed/Deleted");
-
-    if ((context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor")) &&
-        context.config.getBoolean ("fontunderline"))
-    {
-      table.setColumnUnderline (0);
-    }
-    else
-      table.setTableDashedUnderline ();
-
-    Color color_add    (context.config.get ("color.history.add"));
-    Color color_done   (context.config.get ("color.history.done"));
-    Color color_delete (context.config.get ("color.history.delete"));
-
-    // Determine the longest line, and the longest "added" line.
-    int maxAddedLine = 0;
-    int maxRemovedLine = 0;
-    foreach (i, groups)
-    {
-      if (completedGroup[i->first] + deletedGroup[i->first] > maxRemovedLine)
-        maxRemovedLine = completedGroup[i->first] + deletedGroup[i->first];
-
-      if (addedGroup[i->first] > maxAddedLine)
-        maxAddedLine = addedGroup[i->first];
-    }
-
-    int maxLine = maxAddedLine + maxRemovedLine;
-    if (maxLine > 0)
-    {
-      unsigned int leftOffset = (widthOfBar * maxAddedLine) / maxLine;
-
-      int totalAdded     = 0;
-      int totalCompleted = 0;
-      int totalDeleted   = 0;
-
-      int priorYear = 0;
-      int row = 0;
-      foreach (i, groups)
-      {
-        row = table.addRow ();
-
-        totalAdded     += addedGroup[i->first];
-        totalCompleted += completedGroup[i->first];
-        totalDeleted   += deletedGroup[i->first];
-
-        Date dt (i->first);
-        int m, d, y;
-        dt.toMDY (m, d, y);
-
-        if (y != priorYear)
-        {
-          table.addCell (row, 0, y);
-          priorYear = y;
-        }
-
-        unsigned int addedBar     = (widthOfBar *     addedGroup[i->first]) / maxLine;
-        unsigned int completedBar = (widthOfBar * completedGroup[i->first]) / maxLine;
-        unsigned int deletedBar   = (widthOfBar *   deletedGroup[i->first]) / maxLine;
-
-        std::string bar = "";
-        if (context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor"))
-        {
-          char number[24];
-          std::string aBar = "";
-          if (addedGroup[i->first])
-          {
-            sprintf (number, "%d", addedGroup[i->first]);
-            aBar = number;
-            while (aBar.length () < addedBar)
-              aBar = " " + aBar;
-          }
-
-          std::string cBar = "";
-          if (completedGroup[i->first])
-          {
-            sprintf (number, "%d", completedGroup[i->first]);
-            cBar = number;
-            while (cBar.length () < completedBar)
-              cBar = " " + cBar;
-          }
-
-          std::string dBar = "";
-          if (deletedGroup[i->first])
-          {
-            sprintf (number, "%d", deletedGroup[i->first]);
-            dBar = number;
-            while (dBar.length () < deletedBar)
-              dBar = " " + dBar;
-          }
-
-          bar += std::string (leftOffset - aBar.length (), ' ');
-          bar += color_add.colorize    (aBar);
-          bar += color_done.colorize   (cBar);
-          bar += color_delete.colorize (dBar);
-        }
-        else
-        {
-          std::string aBar = ""; while (aBar.length () < addedBar)     aBar += "+";
-          std::string cBar = ""; while (cBar.length () < completedBar) cBar += "X";
-          std::string dBar = ""; while (dBar.length () < deletedBar)   dBar += "-";
-
-          bar += std::string (leftOffset - aBar.length (), ' ');
-          bar += aBar + cBar + dBar;
-        }
-
-        table.addCell (row, 1, bar);
-      }
-    }
-
-    std::stringstream out;
-    if (table.rowCount ())
-    {
-      out << optionalBlankLine ()
-          << table.render ()
-          << "\n";
-
-      if (context.config.getBoolean ("color") || context.config.getBoolean ("_forcecolor"))
-        out << "Legend: "
-            << color_add.colorize ("added")
-            << ", "
-            << color_done.colorize ("completed")
-            << ", "
-            << color_delete.colorize ("deleted")
-            << optionalBlankLine ()
-            << "\n";
-      else
-        out << "Legend: + added, X completed, - deleted\n";
-    }
-    else
-    {
-      out << "No tasks.\n";
-      rc = 1;
-    }
-
-    outs = out.str ();
-    context.hooks.trigger ("post-ghistory-command");
-  }
-
-  return rc;
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 int handleReportTimesheet (std::string& outs)
 {
@@ -2167,10 +1501,29 @@ int handleReportCalendar (std::string& outs)
       yFrom = oldest.year();
     }
 
+    if (context.config.getBoolean ("calendar.offset"))
+    {
+      int moffset = context.config.getInteger ("calendar.offset.value") % 12;
+      int yoffset = context.config.getInteger ("calendar.offset.value") / 12;
+      mFrom += moffset;
+      yFrom += yoffset;
+      if (mFrom < 1)
+      {
+        mFrom += 12;
+        yFrom--;
+      }
+      else if (mFrom > 12)
+      {
+        mFrom -= 12;
+        yFrom++;
+      }
+    }
+
     mTo = mFrom + monthsToDisplay - 1;
     yTo = yFrom;
-    if (mTo > 12) {
-      mTo -=12;
+    if (mTo > 12)
+    {
+      mTo -= 12;
       yTo++;
     }
 
