@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2010, Paul Beckingham.
+// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify it under
@@ -83,8 +83,10 @@ int confirm3 (const std::string& question)
 {
   std::vector <std::string> options;
   options.push_back ("Yes");
+  options.push_back ("yes");
   options.push_back ("no");
   options.push_back ("All");
+  options.push_back ("all");
 
   std::string answer;
   std::vector <std::string> matches;
@@ -93,9 +95,9 @@ int confirm3 (const std::string& question)
   {
     std::cout << question
               << " ("
-              << options[0] << "/"
               << options[1] << "/"
-              << options[2]
+              << options[2] << "/"
+              << options[4]
               << ") ";
 
     std::getline (std::cin, answer);
@@ -105,7 +107,9 @@ int confirm3 (const std::string& question)
   while (matches.size () != 1);
 
        if (matches[0] == "Yes") return 1;
+  else if (matches[0] == "yes") return 1;
   else if (matches[0] == "All") return 2;
+  else if (matches[0] == "all") return 2;
   else                          return 0;
 }
 
@@ -118,8 +122,10 @@ int confirm4 (const std::string& question)
 {
   std::vector <std::string> options;
   options.push_back ("Yes");
+  options.push_back ("yes");
   options.push_back ("no");
   options.push_back ("All");
+  options.push_back ("all");
   options.push_back ("quit");
 
   std::string answer;
@@ -129,10 +135,10 @@ int confirm4 (const std::string& question)
   {
     std::cout << question
               << " ("
-              << options[0] << "/"
               << options[1] << "/"
               << options[2] << "/"
-              << options[3]
+              << options[4] << "/"
+              << options[5]
               << ") ";
 
     std::getline (std::cin, answer);
@@ -142,7 +148,9 @@ int confirm4 (const std::string& question)
   while (matches.size () != 1);
 
        if (matches[0] == "Yes")  return 1;
+  else if (matches[0] == "yes")  return 1;
   else if (matches[0] == "All")  return 2;
+  else if (matches[0] == "all")  return 2;
   else if (matches[0] == "quit") return 3;
   else                           return 0;
 }
@@ -435,11 +443,130 @@ std::string taskDifferences (const Task& before, const Task& after)
   return out.str ();
 }
 
+////////////////////////////////////////////////////////////////////////////////
+std::string taskInfoDifferences (const Task& before, const Task& after)
+{
+  // Attributes are all there is, so figure the different attribute names
+  // between before and after.
+  std::vector <std::string> beforeAtts;
+  foreach (att, before)
+    beforeAtts.push_back (att->first);
+
+  std::vector <std::string> afterAtts;
+  foreach (att, after)
+    afterAtts.push_back (att->first);
+
+  std::vector <std::string> beforeOnly;
+  std::vector <std::string> afterOnly;
+  listDiff (beforeAtts, afterAtts, beforeOnly, afterOnly);
+
+  // Now start generating a description of the differences.
+  std::stringstream out;
+  foreach (name, beforeOnly)
+  {
+    if (*name == "depends")
+    {
+        std::vector <int> deps_before;
+        before.getDependencies (deps_before);
+        std::string from;
+        join (from, ",", deps_before);
+
+        out << "dependencies '"
+            << from
+            << "' deleted\n";
+    }
+    else if (name->substr (0, 11) == "annotation_")
+    {
+      out << "annotation '"
+          << before.get (*name)
+          << "' deleted";
+    }
+    else
+    {
+      out << *name
+          << " deleted\n";
+    }
+  }
+
+  foreach (name, afterOnly)
+  {
+    if (*name == "depends")
+    {
+      std::vector <int> deps_after;
+      after.getDependencies (deps_after);
+      std::string to;
+      join (to, ",", deps_after);
+
+      out << *name
+          << " set to '"
+          << to
+          << "'\n";
+    }
+    else if (name->substr (0, 11) == "annotation_")
+    {
+      out << "annotation added '"
+          << after.get (*name)
+          << "'\n";
+    }
+    else
+      out << *name
+          << " set to '"
+          << renderAttribute (*name, after.get (*name))
+          << "'\n";
+  }
+
+  foreach (name, beforeAtts)
+    if (*name              != "uuid" &&
+        before.get (*name) != after.get (*name))
+    {
+      if (*name == "depends")
+      {
+        std::vector <int> deps_before;
+        before.getDependencies (deps_before);
+        std::string from;
+        join (from, ",", deps_before);
+
+        std::vector <int> deps_after;
+        after.getDependencies (deps_after);
+        std::string to;
+        join (to, ",", deps_after);
+
+        out << *name
+            << " changed from '"
+            << from
+            << "' to '"
+            << to
+            << "'\n";
+      }
+      else if (name->substr (0, 11) == "annotation_")
+      {
+        out << "annotation '"
+            << after.get (*name)
+            << "'\n";
+      }
+      else
+        out << *name
+            << " changed from '"
+            << renderAttribute (*name, before.get (*name))
+            << "' to '"
+            << renderAttribute (*name, after.get (*name))
+            << "'\n";
+    }
+
+
+  // Shouldn't just say nothing.
+  if (out.str ().length () == 0)
+    out << "No changes made.\n";
+
+  return out.str ();
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 std::string renderAttribute (const std::string& name, const std::string& value)
 {
   Att a;
-  if (a.type (name) == "date")
+  if (a.type (name) == "date" &&
+      value != "")
   {
     Date d ((time_t)::atoi (value.c_str ()));
     return d.toString (context.config.get ("dateformat"));
